
// (C) 1997 Glynn Clements <glynn@sensei.co.uk> - Freely Redistributable

package java.awt;

public class BorderLayout implements LayoutManager
{
	private Component[] members;
	private int hgap, vgap;

	private final static String[] names = {
		"North", "South", "East", "West", "Center"
	};

	public String toString()
	{
		return super.toString();
	}

	public void addLayoutComponent(String name, Component comp)
	{
		for (int i = 0; i < 5; i++)
			if (names[i].equals(name))
				members[i] = comp;
	}

	public void removeLayoutComponent(Component comp)
	{
		for (int i = 0; i < 5; i++)
			if (members[i] == comp)
				members[i] = null;
	}

	public void layoutContainer(Container target)
	{
		// TODO: implement this properly

		Dimension bbox = target.size();
		Insets insets = target.insets();

		int x0 = insets.left;
		int y0 = insets.top;
		int x1 = bbox.width  - insets.right;
		int y1 = bbox.height - insets.bottom;

		int h0 = members[0].preferredSize().height;
		members[0].reshape(
			x0,
			y0,
			x1 - x0,
			h0
		);

		int h1 = members[1].preferredSize().height;
		members[1].reshape(
			x0,
			y1 - h1,
			x1 - x0,
			h1
		);

		int h = y1 - y0 - h0 - h1 - 2 * vgap;

		int w2 = members[2].preferredSize().width;
		members[2].reshape(
			x0,
			y0 + h0 + vgap,
			w2,
			h
		);

		int w3 = members[3].preferredSize().width;
		members[3].reshape(
			x1 - w3,
			y0 + h0 + vgap,
			w3,
			h
		);

		int w = y1 - y0 - w2 - w3 - 2 * hgap;
		members[4].reshape(
			x0 + w2 + hgap,
			y0 + h0 + vgap,
			w2,
			h
		);
	}

	public Dimension minimumLayoutSize(Container target)
	{
		Dimension sizes[] = new Dimension[5];
		for (int i = 0; i < 5; i++)
			sizes[i] = members[i].minimumSize();

		int w0 = sizes[2].width + sizes[4].width + sizes[3].width;
		int h0 = Math.max(sizes[2].height, sizes[3].height);
		h0 = Math.max(h0, sizes[4].height);

		int w = Math.max(w0, Math.max(sizes[0].width, sizes[1].width));
		int h = sizes[0].height + h0 + sizes[1].height;

		return new Dimension(w, h);
	}

	public Dimension preferredLayoutSize(Container target)
	{
		Dimension sizes[] = new Dimension[5];
		for (int i = 0; i < 5; i++)
			sizes[i] = members[i].preferredSize();

		int w0 = sizes[2].width + sizes[4].width + sizes[3].width;
		int h0 = Math.max(sizes[2].height, sizes[3].height);
		h0 = Math.max(h0, sizes[4].height);

		int w = Math.max(w0, Math.max(sizes[0].width, sizes[1].width));
		int h = sizes[0].height + h0 + sizes[1].height;

		return new Dimension(w, h);
	}

	public BorderLayout(int hgap, int vgap)
	{
		this.hgap = hgap;
		this.vgap = vgap;
		members = new Component[5];
	}

	public BorderLayout()
	{
		this(0, 0);
	}
}

