
// (C) 1997 Glynn Clements <glynn@sensei.co.uk> - Freely Redistributable

package java.awt;

import java.util.Vector;
import java.util.Hashtable;
import java.util.Enumeration;

public class CardLayout implements LayoutManager
{
	private Hashtable members;
	private int index;

	private int hgap;
	private int vgap;

	public String toString()
	{
		return super.toString();
	}

	public void addLayoutComponent(String name, Component comp)
	{
		members.put(name, comp);
	}

	public void removeLayoutComponent(Component comp)
	{
		Enumeration e = members.keys();
		while (e.hasMoreElements())
		{
			String name = (String) e.nextElement();
			Component c = (Component) members.get(name);
			if (c != comp)
				continue;
			members.remove(name);
			break;
		}
	}

	private void select(int n, Container target)
	{
		target.getComponent(index).hide();
		index = n;
		target.getComponent(index).show();
		target.validate();
	}

	public void show(Container target, String name)
	{
		Component comp = (Component) members.get(name);
		if (comp == null)
			return;

		int n = target.countComponents();
		for (int i = 0; i < n; i++)
		{
			if (target.getComponent(i) != comp)
				continue;
			select(i, target);
			break;
		}
	}

	public void first(Container target)
	{
		select(0, target);
	}

	public void last(Container target)
	{
		select(target.countComponents() - 1, target);
	}

	public void next(Container target)
	{
		int n = (index + 1) % target.countComponents();
		select(n, target);
	}

	public void previous(Container target)
	{
		int n = (index > 0) ? index - 1 : target.countComponents() - 1;
		select(n, target);
	}

	public Dimension minimumLayoutSize(Container target)
	{
		Dimension result = new Dimension(0, 0);

		int n = target.countComponents();
		for (int i = 0; i < n; i++)
		{
			Dimension d = target.getComponent(i).minimumSize();
			if (result.width < d.width)
				result.width = d.width;
			if (result.height < d.height)
				result.height = d.height;
		}

		Insets insets = target.insets();
		result.width  += 2 * hgap + insets.left + insets.right;
		result.height += 2 * vgap + insets.top + insets.bottom;

		return result;
	}

	public Dimension preferredLayoutSize(Container target)
	{
		Dimension result = new Dimension(0, 0);

		int n = target.countComponents();
		for (int i = 0; i < n; i++)
		{
			Dimension d = target.getComponent(i).preferredSize();
			if (result.width < d.width)
				result.width = d.width;
			if (result.height < d.height)
				result.height = d.height;
		}

		Insets insets = target.insets();
		result.width  += 2 * hgap + insets.left + insets.right;
		result.height += 2 * vgap + insets.top + insets.bottom;

		return result;
	}

	public void layoutContainer(Container target)
	{
		Insets insets = target.insets();

		int x0 = insets.left	+ hgap;
		int y0 = insets.top	+ vgap;
		int x1 = insets.right	- hgap;
		int y1 = insets.bottom	- vgap;
		int w = x1 - x0;
		int h = y1 - y0;

		int n = target.countComponents();
		for (int i = 0; i < n; i++)
			target.getComponent(i).reshape(x0, y0, w, h);
	}

	public CardLayout(int hgap, int vgap)
	{
		this.hgap = hgap;
		this.vgap = vgap;
	}

	public CardLayout()
	{
		this(0, 0);
	}
}

