
// (C) 1997 Glynn Clements <glynn@sensei.co.uk> - Freely Redistributable

package java.awt;

public final class Color
{
	public final static Color black		= new Color(  0,   0,   0);
	public final static Color darkGray	= new Color( 64,  64,  64);
	public final static Color gray		= new Color(128, 128, 128);
	public final static Color lightGray	= new Color(192, 192, 192);
	public final static Color white		= new Color(255, 255, 255);
	public final static Color red		= new Color(255,   0,   0);
	public final static Color yellow	= new Color(255, 255,   0);
	public final static Color green		= new Color(  0, 255,   0);
	public final static Color cyan		= new Color(  0, 255, 255);
	public final static Color blue		= new Color(  0,   0, 255);
	public final static Color magenta	= new Color(255,   0, 255);
	public final static Color orange	= new Color(255, 200,   0);
	public final static Color pink		= new Color(255, 175, 175);

	private int data;
	private long nativeData;

	private static int f2i(float f)
	{
		return (int)(255.9999 * f);
	}

	private static int rgb2i(int r, int g, int b)
	{
		return (r << 16) | (g << 8) | b;
	}

	public static Color getColor(String name)
	{
		Integer data = Integer.getInteger(name);
		return (data != null) ? new Color(data.intValue()) : null;
	}

	public static Color getColor(String name, Color v)
	{
		Integer data = Integer.getInteger(name);
		return (data != null) ? new Color(data.intValue()) : v;
	}

	public static Color getColor(String name, int v)
	{
		Integer data = Integer.getInteger(name);
		return (data != null) ? new Color(data.intValue()) : new Color(v);
	}

	public static float[] RGBtoHSB(int r, int g, int b, float hsb[])
	{
		// from Foley, van Dam, Feiner, Hughes (2nd Ed) p592

		int min = Math.min(r, Math.min(g, b));
		int max = Math.max(r, Math.max(g, b));

		float v = max / 255.0f;

		float s = (max == 0) ? 0 : (max - min) * 1.0f / max;

		int delta = max - min;

		int hue	= (max == r) ? (g - b) * 1000 / delta + 6000
			: (max == g) ? (b - r) * 1000 / delta + 2000
			: (max == b) ? (r - g) * 1000 / delta + 4000
			: 0;

		float h = (hue % 6000) / 6000.0f;

		if (hsb == null)
			hsb = new float[3];

		hsb[0] = h;
		hsb[1] = s;
		hsb[2] = v;

		return hsb;
	}

	public static int HSBtoRGB(float h, float s, float b)
	{
		// from Foley, van Dam, Feiner, Hughes (2nd Ed) p593

		float f = (h * 6) % 1.0f;

		int v = f2i(b);
		int p = f2i(b * (1 - s));
		int q = f2i(b * (1 - (s * f)));
		int t = f2i(b * (1 - (s * (1 - f))));

		int i = (int)(h * 6) % 6;

		switch (i)
		{
		case 0:	return rgb2i(v, t, p);
		case 1:	return rgb2i(q, v, p);
		case 2:	return rgb2i(p, v, t);
		case 3:	return rgb2i(p, q, v);
		case 4:	return rgb2i(t, p, v);
		case 5:	return rgb2i(v, p, q);
		default: throw new RuntimeException("reality failure");
		}
	}

	public static Color getHSBColor(float h, float s, float b)
	{
		int data = HSBtoRGB(h, s, b);
		return new Color(data);
	}

	public int hashCode()
	{
		return data;
	}

	public boolean equals(Object obj)
	{
		if (obj == null)
			return false;
		if (!(obj instanceof Color))
			return false;

		Color p = (Color) obj;

		return p.getRGB() == data;
	}

	public String toString()
	{
		return	"(" + 
			getRed() + ", " + 
			getGreen() + ", " + 
			getBlue() + ")";
	}

	public Color brighter()
	{
		int r = (1 + getRed())	* 7 / 8;
		int g = (1 + getGreen())* 7 / 8;
		int b = (1 + getBlue())	* 7 / 8;
		return new Color(r, g, b);
	}

	public Color darker()
	{
		int r = getRed()	* 7 / 8;
		int g = getGreen()	* 7 / 8;
		int b = getBlue()	* 7 / 8;
		return new Color(r, g, b);
	}

	public int getRed()
	{
		return (data >> 16) & 0xFF;
	}

	public int getGreen()
	{
		return (data >> 8) & 0xFF;
	}

	public int getBlue()
	{
		return data & 0xFF;
	}

	public int getRGB()
	{
		return data;
	}

	public Color(int rgb)
	{
		data = rgb;
	}

	public Color(int r, int g, int b)
	{
		this(rgb2i(r, g, b));
	}

	public Color(float r, float g, float b)
	{
		this(f2i(r), f2i(g), f2i(b));
	}
}

