
// (C) 1997 Glynn Clements <glynn@sensei.co.uk> - Freely Redistributable

package java.awt;

public class FlowLayout implements LayoutManager
{
	public final static int LEFT	= 0;
	public final static int CENTER	= 1;
	public final static int RIGHT	= 2;

	private int align;
	private int hgap;
	private int vgap;

	public String toString()
	{
		return super.toString();
	}

	public void addLayoutComponent(String name, Component comp)
	{
		// this function is supposed to be empty
	}

	public void removeLayoutComponent(Component comp)
	{
		// this function is supposed to be empty
	}

	public Dimension minimumLayoutSize(Container target)
	{
		Component[] members = target.getComponents();

		int w = 0;
		int h = 0;

		for (int i = 0; i < members.length; i++)
		{
			Dimension d = members[i].minimumSize();
			w += d.width + hgap;
			if (h < d.height)
				h = d.height;
		}

		Insets insets = target.insets();
		w += insets.left + insets.right + hgap;
		h += insets.top + insets.bottom + 2 * vgap;

		return new Dimension(w, h);
	}

	public Dimension preferredLayoutSize(Container target)
	{
		Component[] members = target.getComponents();

		int w = 0;
		int h = 0;

		for (int i = 0; i < members.length; i++)
		{
			Dimension d = members[i].preferredSize();
			w += d.width + hgap;
			if (h < d.height)
				h = d.height;
		}

		Insets insets = target.insets();
		w += insets.left + insets.right + hgap;
		h += insets.top + insets.bottom + 2 * vgap;

		return new Dimension(w, h);
	}

	public void layoutContainer(Container target)
	{
		// TODO: implement the align option

		Component[] members = target.getComponents();

		Dimension bbox = target.size();

		Insets insets = target.insets();
		int x = insets.left + hgap;
		int y = insets.top + vgap;
		int h = 0;

		for (int i = 0; i < members.length; i++)
		{
			Dimension d = members[i].preferredSize();
			int nx = x + d.width + hgap;
			if (nx > bbox.width && h > 0)
			{
				x = insets.left + hgap;
				y += h;
				h = 0;
			}
			members[i].reshape(x, y, d.width, d.height);
			if (h < d.height)
				h = d.height;
			x = nx;
		}
	}

	public FlowLayout(int align, int hgap, int vgap)
	{
		this.align = align;
		this.hgap = hgap;
		this.vgap = vgap;
	}

	public FlowLayout(int align)
	{
		this(align, 5, 5);
	}

	public FlowLayout()
	{
		this(CENTER, 5, 5);
	}
}

