
// (C) 1997 Glynn Clements <glynn@sensei.co.uk> - Freely Redistributable

package java.awt;

public class Font
{
	public final static int PLAIN	= 0;
	public final static int BOLD	= 1;
	public final static int ITALIC	= 2;

	private int nativeData;

	protected String name;
	protected int style;
	protected int size;

	private final static String[] styleNames = {
		"", "bold", "italic", "bolditalic"
	};

	public static Font getFont(String prop, Font font)
	{
		String string = System.getProperty(prop);
		if (string == null)
			return font;

		String name;
		int style = PLAIN;
		int size = 12;

		int n = string.indexOf('-');
		if (n >= 0)
		{
			name = string.substring(0, n);
			string = string.substring(n + 1);

			if (string.startsWith("bolditalic"))
				style = BOLD | ITALIC;
			else if (string.startsWith("italic"))
				style = ITALIC;
			else if (string.startsWith("bold"))
				style = BOLD;

			n = string.indexOf('-');
			if (n >= 0)
			{
				string = string.substring(n + 1);
				size = Integer.parseInt(string);
			}
		}
		else
			name = string;

		return new Font(name, style, size);
	}

	public static Font getFont(String name)
	{
		return getFont(name, null);
	}

	public int hashCode()
	{
		return name.hashCode() ^ (style << 30) ^ size;
	}

	public String toString()
	{
		return name + "-" + styleNames[style] + "-" + size;
	}

	public boolean equals(Object obj)
	{
		if (obj == null)
			return false;
		if (!(obj instanceof Font))
			return false;

		Font f = (Font) obj;

		return (
			f.name.equals(name) &&
			f.style == style &&
			f.size == size
		);
	}

	public String getFamily()
	{
		return name;
	}

	public String getName()
	{
		return name;
	}

	public int getStyle()
	{
		return style;
	}

	public int getSize()
	{
		return size;
	}

	public boolean isBold()
	{
		return (style & BOLD) != 0;
	}

	public boolean isItalic()
	{
		return (style & ITALIC) != 0;
	}

	public boolean isPlain()
	{
		return style == 0;
	}

	public Font(String name, int style, int size)
	{
		this.name = name;
		this.style = style;
		this.size = size;
	}
}
