
// (C) 1997 Glynn Clements <glynn@sensei.co.uk> - Freely Redistributable

package java.awt;

import java.awt.image.ImageObserver;

public abstract class Graphics
{
	public abstract Graphics create();
	public abstract void dispose();
	public abstract FontMetrics getFontMetrics(Font f);
	public abstract void translate(int x, int y);
	public abstract void clipRect(int x, int y, int w, int h);
	public abstract Rectangle getClipRect();
	public abstract void setColor(Color c);
	public abstract Color getColor();
	public abstract void setFont(Font font);
	public abstract Font getFont();
	public abstract void setXORMode(Color c1);
	public abstract void setPaintMode();
	public abstract void drawString(String str, int x, int y);
	public abstract boolean drawImage(Image img, int x, int y, Color bgcolor, ImageObserver observer);
	public abstract boolean drawImage(Image img, int x, int y, ImageObserver observer);
	public abstract boolean drawImage(Image img, int x, int y, int w, int h, Color bgcolor, ImageObserver observer);
	public abstract boolean drawImage(Image img, int x, int y, int w, int h, ImageObserver observer);
	public abstract void drawLine(int x1, int y1, int x2, int y2);
	public abstract void drawPolygon(int xPoints[], int yPoints[], int nPoints);
	public abstract void drawArc(int x, int y, int w, int h, int startAngle, int arcAngle);
	public abstract void drawOval(int x, int y, int w, int h);
	public abstract void drawRoundRect(int x, int y, int w, int h, int arcWidth, int arcHeight);
	public abstract void fillPolygon(int xPoints[], int yPoints[], int nPoints);
	public abstract void fillArc(int x, int y, int w, int h, int startAngle, int arcAngle);
	public abstract void fillOval(int x, int y, int w, int h);
	public abstract void fillRect(int x, int y, int w, int h);
	public abstract void fillRoundRect(int x, int y, int w, int h, int arcWidth, int arcHeight);
	public abstract void clearRect(int x, int y, int w, int h);
	public abstract void copyArea(int x, int y, int w, int h, int dx, int dy);

	public String toString()
	{
		return super.toString();
	}

	public void finalize()
	{
		dispose();
	}

	public Graphics create(int x, int y, int w, int h)
	{
		Graphics g = create();
		g.translate(x, y);
		g.clipRect(0, 0, w, h);
		return g;
	}

	public FontMetrics getFontMetrics()
	{
		return getFontMetrics(getFont());
	}

	public void drawBytes(byte data[], int offset, int length, int x, int y)
	{
		String string = new String(data, 0, offset, length);
		drawString(string, x, y);
	}

	public void drawChars(char data[], int offset, int length, int x, int y)
	{
		String string = new String(data, offset, length);
		drawString(string, x, y);
	}

	public void drawRect(int x, int y, int w, int h)
	{
		drawLine(x + 0, y + 0, x + w, y + 0);
		drawLine(x + w, y + 0, x + w, y + h);
		drawLine(x + w, y + h, x + 0, y + h);
		drawLine(x + 0, y + h, x + 0, y + 0);
	}

	public void draw3DRect(int x, int y, int w, int h, boolean raised)
	{
		fill3DRect(x, y, w, h, raised);
	}

	public void drawPolygon(Polygon p)
	{
		drawPolygon(p.xpoints, p.ypoints, p.npoints);
	}

	public void fill3DRect(int x, int y, int w, int h, boolean raised)
	{
		int[] xp = new int[3];
		int[] yp = new int[3];

		Color colour = getColor();
		Color hilite = colour.brighter();
		Color lolite = colour.darker();

		xp[0] = x + w;	yp[0] = y;
		xp[1] = x;	yp[1] = y + h;
		xp[2] = x;	yp[2] = y;
		setColor(raised ? hilite : lolite);
		fillPolygon(xp, yp, 3);

		xp[0] = x + w;	yp[0] = y;
		xp[1] = x + w;	yp[1] = y + h;
		xp[2] = x;	yp[2] = y + h;
		setColor(raised ? lolite : hilite);
		fillPolygon(xp, yp, 3);

		setColor(colour);
		fillRect(x, y, w, h);
	}

	public void fillPolygon(Polygon p)
	{
		fillPolygon(p.xpoints, p.ypoints, p.npoints);
	}

	protected Graphics()
	{
	}
}

