
// (C) 1997 Glynn Clements <glynn@sensei.co.uk> - Freely Redistributable

package java.awt;

public class GridLayout implements LayoutManager
{
	private int rows;
	private int cols;

	private int hgap;
	private int vgap;

	private boolean rowsAuto;
	private boolean colsAuto;

	private void autoShape(int n)
	{
		if (rowsAuto)
			rows = (n + cols - 1) / cols;
		if (colsAuto)
			cols = (n + rows - 1) / rows;
	}

	public String toString()
	{
		return super.toString();
	}

	public void addLayoutComponent(String name, Component comp)
	{
		// this function is supposed to be empty
	}

	public void removeLayoutComponent(Component comp)
	{
		// this function is supposed to be empty
	}

	public Dimension minimumLayoutSize(Container target)
	{
		Component[] members = target.getComponents();

		autoShape(members.length);

		int w = 0;
		int h = 0;

		for (int i = 0; i < members.length; i++)
		{
			Dimension d = members[i].minimumSize();
			if (w < d.width)
				w = d.width;
			if (h < d.height)
				h = d.height;
		}

		w *= rows;
		h *= cols;

		Insets insets = target.insets();
		w += insets.left + insets.right + (cols + 1) * hgap;
		h += insets.top + insets.bottom + (rows + 1) * vgap;

		return new Dimension(w, h);
	}

	public Dimension preferredLayoutSize(Container target)
	{
		Component[] members = target.getComponents();

		autoShape(members.length);

		int w = 0;
		int h = 0;

		for (int i = 0; i < members.length; i++)
		{
			Dimension d = members[i].preferredSize();
			if (w < d.width)
				w = d.width;
			if (h < d.height)
				h = d.height;
		}

		w *= rows;
		h *= cols;

		Insets insets = target.insets();
		w += insets.left + insets.right + (cols + 1) * hgap;
		h += insets.top + insets.bottom + (rows + 1) * vgap;

		return new Dimension(w, h);
	}

	public void layoutContainer(Container target)
	{
		Dimension d = target.size();
		int w = d.width;
		int h = d.height;

		Insets insets = target.insets();
		w -= insets.left + insets.right + (cols + 1) * hgap;
		h -= insets.top + insets.bottom + (rows + 1) * vgap;

		Component[] members = target.getComponents();

		autoShape(members.length);

		w /= cols;
		h /= rows;

		int x = insets.left + hgap;
		int y = insets.top  + vgap;

		int n = 0;
	loop:
		for (int i = 0; i < rows; i++)
		{
			for (int j = 0; j < cols; j++)
			{
				if (n >= members.length)
					break loop;
				members[n].reshape(x, y, w + hgap, h + vgap);
				x += w;
				y += h;
				n++;
			}
		}
	}

	public GridLayout(int rows, int cols, int hgap, int vgap)
	{
		this.rows = rows;
		this.cols = cols;
		this.hgap = hgap;
		this.vgap = vgap;
		rowsAuto = (rows == 0);
		colsAuto = (cols == 0);
	}

	public GridLayout(int rows, int cols)
	{
		this(rows, cols, 0, 0);
	}
}
