
// (C) 1997 Glynn Clements <glynn@sensei.co.uk> - Freely Redistributable

package java.awt;

import java.awt.peer.MenuPeer;

import java.util.Vector;

public class Menu extends MenuItem implements MenuContainer
{
	private Vector items;
	private String label;
	private boolean tearOff;

	public void addNotify()
	{
		MenuBar menubar = (MenuBar) this.getParent();
		Window window = (Window) menubar.getParent();
		Toolkit toolkit = window.getToolkit();

		peer = toolkit.createMenu(this);
	}

	public void removeNotify()
	{
		int n = countItems();
		for (int i = 0; i < n; i++)
		{
			MenuItem item = (MenuItem) items.elementAt(i);
			item.removeNotify();
		}
		if (peer != null)
			peer.dispose();
	}

	public MenuItem add(MenuItem item)
	{
		if (item.getParent() != null)
			item.getParent().remove(item);

		items.addElement(item);
		if (peer != null)
			((MenuPeer) peer).addItem(item);

		return item;
	}

	public void add(String label)
	{
		MenuItem item = new MenuItem(label);
		items.addElement(item);
		if (peer != null)
			((MenuPeer) peer).addItem(item);
	}

	public void addSeparator()
	{
		if (peer != null)
			((MenuPeer) peer).addSeparator();
	}

	public void remove(int index)
	{
		items.removeElementAt(index);
		if (peer != null)
			((MenuPeer) peer).delItem(index);
	}

	public void remove(MenuComponent item)
	{
		int n = items.indexOf(item);
		if (n >= 0)
			remove(n);
	}

	public int countItems()
	{
		return items.size();
	}

	public MenuItem getItem(int index)
	{
		return (MenuItem) items.elementAt(index);
	}

	public boolean isTearOff()
	{
		return tearOff;
	}

	public Menu(String label, boolean tearOff)
	{
		super(label);
		this.tearOff = tearOff;
	}

	public Menu(String label)
	{
		this(label, false);
	}
}

