
// (C) 1997 Glynn Clements <glynn@sensei.co.uk> - Freely Redistributable

package java.awt;

public class Polygon
{
	private Rectangle bbox = null;

	public int npoints;
	public int xpoints[];
	public int ypoints[];

	private void ensureSize()
	{
		if (npoints < xpoints.length && npoints < ypoints.length)
			return;

		int[] xp = new int[npoints * 2];
		int[] yp = new int[npoints * 2];
		System.arraycopy(xpoints, 0, xp, 0, npoints);
		System.arraycopy(ypoints, 0, yp, 0, npoints);
		xpoints = xp;
		ypoints = yp;
	}

	public void addPoint(int x, int y)
	{
		ensureSize();
		xpoints[npoints] = x;
		ypoints[npoints] = y;
		npoints++;
		if (bbox != null)
			getBoundingBox();
	}

	public Rectangle getBoundingBox()
	{
		bbox = new Rectangle();
		if (npoints > 0)
			bbox.move(xpoints[0], ypoints[0]);
		for (int i = 1; i < npoints; i++)
			bbox.add(xpoints[i], ypoints[i]);
		return bbox;
	}

	public boolean inside(int x, int y)
	{
		// TODO: Make this function work for non-convex polygons

		if (npoints < 3)
			return false;

		if (bbox == null)
			getBoundingBox();

		if (!bbox.inside(x, y))
			return false;

		int x0 = xpoints[0];
		int y0 = ypoints[0];

		for (int i = 1; i < npoints; i++)
		{
			int x1 = xpoints[i];
			int y1 = ypoints[i];

			int vx = x1 - x0;
			int vy = y1 - y0;

			int nx = -vy;
			int ny =  vx;

			int dx = x - x0;
			int dy = y - y0;

			int k = nx * dx + ny * dy;

			boolean inside = k > 0;

			if (!inside)
				return false;

			x0 = x1;
			y0 = y1;
		}

		return true;
	}

	public Polygon(int xp[], int yp[], int npoints)
	{
		xpoints = new int[npoints];
		ypoints = new int[npoints];
		System.arraycopy(xp, 0, xpoints, 0, npoints);
		System.arraycopy(yp, 0, ypoints, 0, npoints);
	}

	public Polygon()
	{
		xpoints = new int[8];
		ypoints = new int[8];
	}
}

