
// (C) 1997 Glynn Clements <glynn@sensei.co.uk> - Freely Redistributable

package java.awt;

public class Rectangle
{
	public int width;
	public int height;
	public int x;
	public int y;

	public int hashCode()
	{
		return x ^ y ^ width ^ height;
	}

	public String toString()
	{
		return "" + width + "x" + height + "+" + x + "+" + y;
	}

	public boolean equals(Object obj)
	{
		if (obj == null)
			return false;
		if (!(obj instanceof Rectangle))
			return false;

		Rectangle r = (Rectangle) obj;

		return	r.x == x &&
			r.y == y &&
			r.width == width &&
			r.height == height;
	}

	public boolean isEmpty()
	{
		return	width <= 0 || height <= 0;
	}

	public boolean inside(int px, int py)
	{
		return	px >= x &&
			py >= y &&
			px < x + width &&
			py < y + height;
	}

	public Rectangle intersection(Rectangle r)
	{
		int x0 = Math.max(r.x, x);
		int y0 = Math.max(r.y, y);
		int x1 = Math.min(r.x + r.width , x + width);
		int y1 = Math.min(r.y + r.height, y + height);
		return new Rectangle(x0, y0, x1 - x0, y1 - y0);
	}

	public Rectangle union(Rectangle r)
	{
		int x0 = Math.min(r.x, x);
		int y0 = Math.min(r.y, y);
		int x1 = Math.max(r.x + r.width , x + width);
		int y1 = Math.max(r.y + r.height, y + height);
		return new Rectangle(x0, y0, x1 - x0, y1 - y0);
	}

	public boolean intersects(Rectangle r)
	{
		return !intersection(r).isEmpty();
	}

	public void add(int px, int py)
	{
		int x0 = Math.min(px, x);
		int y0 = Math.min(py, y);
		int x1 = Math.max(px, x + width);
		int y1 = Math.max(py, y + height);
		x = x0;
		y = y0;
		width  = x1 - x0;
		height = y1 - y0;
	}

	public void add(Point p)
	{
		add(p.x, p.y);
	}

	public void add(Rectangle r)
	{
		int x0 = Math.min(r.x, x);
		int y0 = Math.min(r.y, y);
		int x1 = Math.max(r.x + r.width,  x + width);
		int y1 = Math.max(r.y + r.height, y + height);
		x = x0;
		y = y0;
		width  = x1 - x0;
		height = y1 - y0;
	}

	public void grow(int dx, int dy)
	{
		x -= dx;
		y -= dy;
		width  += 2 * dx;
		height += 2 * dy;
	}

	public void move(int x, int y)
	{
		this.x = x;
		this.y = y;
	}

	public void translate(int dx, int dy)
	{
		move(x + dx, y + dy);
	}

	public void resize(int width, int height)
	{
		this.width  = width;
		this.height = height;
	}

	public void reshape(int x, int y, int width, int height)
	{
		move(x, y);
		resize(width, height);
	}

	public Rectangle(int x, int y, int width, int height)
	{
		reshape(x, y, width, height);
	}

	public Rectangle(Point p, Dimension d)
	{
		this(p.x, p.y, d.width, d.height);
	}

	public Rectangle(int width, int height)
	{
		this(0, 0, width, height);
	}

	public Rectangle(Dimension d)
	{
		this(0, 0, d.width, d.height);
	}

	public Rectangle(Point p)
	{
		this(p.x, p.y, 0, 0);
	}

	public Rectangle()
	{
		this(0, 0, 0, 0);
	}
}

