
// (C) 1997 Glynn Clements <glynn@sensei.co.uk> - Freely Redistributable

package java.awt;

import java.net.URL;
import java.awt.image.*;
import java.awt.peer.*;

public abstract class Toolkit
{
	private static Toolkit defaultToolkit;

	public static Toolkit getDefaultToolkit()
	{
		if (defaultToolkit != null)
			return defaultToolkit;

		String name = System.getProperty("awt.toolkit");
		if (name == null)
			name = "sun.awt.motif.MToolkit";

		try
		{
			Class clazz = Class.forName(name);
			defaultToolkit = (Toolkit) clazz.newInstance();
			return defaultToolkit;
		}
		catch (ClassNotFoundException e)
		{
			throw new AWTError("unable to locate Toolkit " + name);
		}
		catch (InstantiationException e)
		{
			throw new AWTError("unable to instantiate Toolkit " + name);
		}
		catch (IllegalAccessException e)
		{
			throw new AWTError("unable to instantiate Toolkit " + name);
		}
	}

	protected abstract CanvasPeer createCanvas(Canvas target);
	protected abstract LabelPeer createLabel(Label target);
	protected abstract ButtonPeer createButton(Button target);
	protected abstract CheckboxPeer createCheckbox(Checkbox target);
	protected abstract ListPeer createList(List target);
	protected abstract ChoicePeer createChoice(Choice target);
	protected abstract ScrollbarPeer createScrollbar(Scrollbar target);
	protected abstract TextFieldPeer createTextField(TextField target);
	protected abstract TextAreaPeer createTextArea(TextArea target);
	protected abstract PanelPeer createPanel(Panel target);
	protected abstract FramePeer createFrame(Frame target);
	protected abstract WindowPeer createWindow(Window target);
	protected abstract DialogPeer createDialog(Dialog target);
	protected abstract FileDialogPeer createFileDialog(FileDialog target);
	protected abstract MenuPeer createMenu(Menu target);
	protected abstract MenuBarPeer createMenuBar(MenuBar target);
	protected abstract MenuItemPeer createMenuItem(MenuItem target);
	protected abstract CheckboxMenuItemPeer createCheckboxMenuItem(CheckboxMenuItem target);
	public abstract Image createImage(ImageProducer producer);
	public abstract boolean prepareImage(Image image, int width, int height, ImageObserver observer);
	public abstract int checkImage(Image image, int width, int height, ImageObserver observer);
	public abstract Image getImage(String filename);
	public abstract Image getImage(URL url);
	public abstract ColorModel getColorModel();
	public abstract String[] getFontList();
	public abstract FontMetrics getFontMetrics(Font font);
	public abstract int getScreenResolution();
	public abstract Dimension getScreenSize();
	public abstract void sync();

	public Toolkit()
	{
	}
}

