
// (C) 1997 Glynn Clements <glynn@sensei.co.uk> - Freely Redistributable

package java.awt.image;

import java.util.Hashtable;
import java.awt.Rectangle;

public class CropImageFilter extends ImageFilter
{
	private Rectangle box;

	public void setDimensions(int w, int h)
	{
		super.setDimensions(box.width, box.height);
	}

	public void setPixels(int x, int y, int w, int h, ColorModel model, byte[] pixels, int off, int scansize)
	{
		Rectangle bounds = new Rectangle(x, y, w, h);
		Rectangle overlap = bounds.intersection(box);
		if (overlap.isEmpty())
			return;
		off += (overlap.y - y) * scansize + (overlap.x - x);
		super.setPixels(overlap.x, overlap.y, overlap.width, overlap.height, model, pixels, off, scansize);
	}

	public void setPixels(int x, int y, int w, int h, ColorModel model, int[] pixels, int off, int scansize)
	{
		Rectangle bounds = new Rectangle(x, y, w, h);
		Rectangle overlap = bounds.intersection(box);
		if (overlap.isEmpty())
			return;
		off += (overlap.y - y) * scansize + (overlap.x - x);
		super.setPixels(overlap.x, overlap.y, overlap.width, overlap.height, model, pixels, off, scansize);
	}

	public void setProperties(Hashtable props)
	{
		props.put("croprect", box);
		super.setProperties(props);
	}

	public CropImageFilter(int x, int y, int w, int h)
	{
		box = new Rectangle(x, y, w, h);
	}
}
