
// (C) 1997 Glynn Clements <glynn@sensei.co.uk> - Freely Redistributable

package java.awt.image;

public class DirectColorModel extends ColorModel
{
	private final int[] shift = new int[4];
	private final int[] mask = new int[4];

	public final int getRGB(int pixel)
	{
		return	(getAlpha(pixel) << 24) +
			(getRed(pixel)   << 16) +
			(getGreen(pixel) <<  8) +
			(getBlue(pixel)  <<  0);
	}

	public final int getRed(int pixel)
	{
		return (pixel >> shift[0]) & mask[0];
	}

	public final int getGreen(int pixel)
	{
		return (pixel >> shift[1]) & mask[1];
	}

	public final int getBlue(int pixel)
	{
		return (pixel >> shift[2]) & mask[2];
	}

	public final int getAlpha(int pixel)
	{
		return	(shift[3] >= 0)
			? (pixel >> shift[3]) & mask[3]
			: 0xFF;
	}

	public final int getRedMask()
	{
		return mask[0];
	}

	public final int getGreenMask()
	{
		return mask[1];
	}

	public final int getBlueMask()
	{
		return mask[2];
	}

	public final int getAlphaMask()
	{
		return mask[3];
	}

	public DirectColorModel(int bits, int rmask, int gmask, int bmask, int amask)
	{
		super(bits);
		mask[0] = rmask;
		mask[1] = gmask;
		mask[2] = bmask;
		mask[3] = amask;

		for (int n = 0; n < 4; n++)
		{
			shift[n] = -1;
			for (int i = 0; i < bits; i++)
			{
				if ((mask[n] & (1 << i)) == 0)
					continue;
				shift[n] = i;
				break;
			}
		}
	}

	public DirectColorModel(int bits, int rmask, int gmask, int bmask)
	{
		this(bits, rmask, gmask, bmask, 0);
	}
}
