
// (C) 1997 Glynn Clements <glynn@sensei.co.uk> - Freely Redistributable

package java.awt.image;

import java.util.Hashtable;

public class FilteredImageSource implements ImageProducer
{
	private Hashtable filters;
	private ImageProducer source;
	private ImageFilter filter;

	private ImageFilter getFilter(ImageConsumer ic)
	{
		return (ImageFilter) filters.get(ic);
	}

	public void addConsumer(ImageConsumer ic)
	{
		ImageFilter instance = filter.getFilterInstance(ic);
		filters.put(ic, instance);
		source.addConsumer(instance);
	}

	public void removeConsumer(ImageConsumer ic)
	{
		ImageFilter instance = getFilter(ic);
		if (instance == null)
			return;
		source.removeConsumer(instance);
	}

	public boolean isConsumer(ImageConsumer ic)
	{
		ImageFilter instance = getFilter(ic);
		return instance != null;
	}

	public void startProduction(ImageConsumer ic)
	{
		ImageFilter instance = filter.getFilterInstance(ic);
		source.startProduction(instance);
	}

	public void requestTopDownLeftRightResend(ImageConsumer ic)
	{
		ImageFilter instance = getFilter(ic);
		source.requestTopDownLeftRightResend(instance);
	}

	public FilteredImageSource(ImageProducer source, ImageFilter filter)
	{
		this.source = source;
		this.filter = filter;
	}
}

