
// (C) 1997 Glynn Clements <glynn@sensei.co.uk> - Freely Redistributable

package java.awt.image;

public class IndexColorModel extends ColorModel
{
	private byte[] r, g, b, a;
	private int size;
	private int trans;

	public final int getRGB(int pixel)
	{
		return	(getAlpha(pixel) << 24) +
			(getRed(pixel)   << 16) +
			(getGreen(pixel) <<  8) +
			(getBlue(pixel)  <<  0);
	}

	public final int getRed(int pixel)
	{
		return r[pixel];
	}

	public final int getGreen(int pixel)
	{
		return g[pixel];
	}

	public final int getBlue(int pixel)
	{
		return b[pixel];
	}

	public final int getAlpha(int pixel)
	{
		return a[pixel];
	}

	public final void getReds(byte[] result)
	{
		System.arraycopy(r, 0, result, 0, size);
	}

	public final void getGreens(byte[] result)
	{
		System.arraycopy(g, 0, result, 0, size);
	}

	public final void getBlues(byte[] result)
	{
		System.arraycopy(b, 0, result, 0, size);
	}

	public final void getAlphas(byte[] result)
	{
		System.arraycopy(a, 0, result, 0, size);
	}

	public final int getMapSize()
	{
		return size;
	}

	public final int getTransparentPixel()
	{
		return trans;
	}

	public IndexColorModel(int bits, int size, byte[] cmap, int start, boolean hasAlpha)
	{
		super(bits);
		this.size = size;

		r = new byte[size];
		g = new byte[size];
		b = new byte[size];
		a = new byte[size];

		for (int n = start, i = 0; i < size; i++)
		{
			r[i] = cmap[n + 0];
			g[i] = cmap[n + 1];
			b[i] = cmap[n + 2];
			a[i] = hasAlpha
				? cmap[n + 3]
				: (byte) 0xFF;
			n += hasAlpha ? 4 : 3;
		}
	}

	public IndexColorModel(int bits, int size, byte[] cmap, int start, boolean hasAlpha, int trans)
	{
		this(bits, size, cmap, start, hasAlpha);
		a[trans] = (byte) 0x00;
	}

	public IndexColorModel(int bits, int size, byte[] r, byte[] g, byte[] b, byte[] a)
	{
		super(bits);
		this.size = size;
		this.trans = -1;

		this.r = new byte[size];
		this.g = new byte[size];
		this.b = new byte[size];
		this.a = new byte[size];

		System.arraycopy(r, 0, this.r, 0, size);
		System.arraycopy(g, 0, this.g, 0, size);
		System.arraycopy(b, 0, this.b, 0, size);
		if (a != null)
			System.arraycopy(a, 0, this.a, 0, size);
	}

	public IndexColorModel(int bits, int size, byte[] r, byte[] g, byte[] b, int trans)
	{
		this(bits, size, r, g, b, null);
		this.trans = trans;
		for (int i = 0; i < size; i++)
			a[i] = (byte) ((i == trans) ? 0x00 : 0xFF);
	}

	public IndexColorModel(int bits, int size, byte[] r, byte[] g, byte[] b)
	{
		this(bits, size, r, g, b, -1);
	}
}
