
// (C) 1997 Glynn Clements <glynn@sensei.co.uk> - Freely Redistributable

package java.awt.image;

import java.util.Hashtable;

public class MemoryImageSource implements ImageProducer
{
	private final static int hints =
		ImageConsumer.SINGLEFRAME |
		ImageConsumer.TOPDOWNLEFTRIGHT |
		ImageConsumer.SINGLEPASS |
		ImageConsumer.COMPLETESCANLINES;

	private ImageConsumer consumer;

	private int width;
	private int height;
	private ColorModel colourModel;
	private byte[] bytes;
	private int[] words;
	private int offset;
	private int scanSize;
	private Hashtable props;

	public void addConsumer(ImageConsumer ic)
	{
		startProduction(ic);
	}

	public boolean isConsumer(ImageConsumer ic)
	{
		return consumer == ic;
	}

	public void removeConsumer(ImageConsumer ic)
	{
		if (consumer == ic)
			consumer = null;
	}

	public void requestTopDownLeftRightResend(ImageConsumer ic)
	{
// 		startProduction(ic);
	}

	public void startProduction(ImageConsumer ic)
	{
		consumer = ic;

		ic.setHints(hints);
		ic.setDimensions(width, height);
		ic.setColorModel(colourModel);
		ic.setProperties(props);
		if (bytes != null)
			ic.setPixels(0, 0, width, height, colourModel, bytes, offset, width);
		else
			ic.setPixels(0, 0, width, height, colourModel, words, offset, width);
		ic.imageComplete(ImageConsumer.STATICIMAGEDONE);
	}

	private MemoryImageSource(int w, int h, ColorModel cm, int off, int scan, Hashtable props)
	{
		width = w;
		height = h;
		colourModel = cm;
		offset = off;
		scanSize = scan;
		props = props;
	}

	public MemoryImageSource(int w, int h, ColorModel cm, byte[] pix, int off, int scan, Hashtable props)
	{
		this(w, h, cm, off, scan, null);
		bytes = pix;
	}

	public MemoryImageSource(int w, int h, ColorModel cm, byte[] pix, int off, int scan)
	{
		this(w, h, cm, pix, off, scan, null);
	}

	public MemoryImageSource(int w, int h, ColorModel cm, int[] pix, int off, int scan, Hashtable props)
	{
		this(w, h, cm, off, scan, null);
		words = pix;
	}

	public MemoryImageSource(int w, int h, ColorModel cm, int[] pix, int off, int scan)
	{
		this(w, h, cm, pix, off, scan, null);
	}

	public MemoryImageSource(int w, int h, int[] pix, int off, int scan, Hashtable props)
	{
		this(w, h, ColorModel.getRGBdefault(), pix, off, scan, null);
	}

	public MemoryImageSource(int w, int h, int[] pix, int off, int scan)
	{
		this(w, h, pix, off, scan, null);
	}
}

