
// (C) 1997 Glynn Clements <glynn@sensei.co.uk> - Freely Redistributable

package java.awt.image;

import java.util.Hashtable;
import java.awt.Image;

public class PixelGrabber implements ImageConsumer
{
	private ImageProducer producer;

	private int x, y, w, h;
	private int[] pixels;
	private int offset;
	private int scanSize;
	private int status;

	public void setHints(int hints)
	{
		// This function is supposed to be empty
	}

	public void setColorModel(ColorModel model)
	{
		// This function is supposed to be empty
	}

	public void setDimensions(int width, int height)
	{
		// This function is supposed to be empty
	}

	public void setProperties(Hashtable props)
	{
		// This function is supposed to be empty
	}

	public void setPixels(int x, int y, int w, int h, ColorModel model, int[] pixels, int offset, int scanSize)
	{
		for (int j = 0; j < h; j++)
		{
			for (int i = 0; i < w; i++)
			{
				int u = x + i;
				int v = y + j;
				int p = u - this.x;
				int q = v - this.y;
				if (p < 0 || q >= this.w || p < 0 || q >= this.h)
					continue;
				int src = pixels[offset + scanSize * j + i];
				int dst = model.getRGB(src);
				this.pixels[this.offset + this.scanSize * q + p] = dst;
			}
		}
	}

	public void setPixels(int x, int y, int w, int h, ColorModel model, byte[] pixels, int offset, int scanSize)
	{
		for (int j = 0; j < h; j++)
		{
			for (int i = 0; i < w; i++)
			{
				int u = x + i;
				int v = y + j;
				int p = u - this.x;
				int q = v - this.y;
				if (p < 0 || q >= this.w || p < 0 || q >= this.h)
					continue;
				byte src = pixels[offset + scanSize * j + i];
				int dst = model.getRGB(src & 0xFF);
				this.pixels[this.offset + this.scanSize * q + p] = dst;
			}
		}
	}

	public synchronized void imageComplete(int status)
	{
		this.status = status;
		notifyAll();
	}

	public synchronized boolean grabPixels() throws InterruptedException
	{
		status = 0;
		producer.startProduction(this);
		wait();
		return (status & (SINGLEFRAMEDONE | STATICIMAGEDONE)) != 0;
	}

	public synchronized boolean grabPixels(long delay) throws InterruptedException
	{
		status = 0;
		producer.startProduction(this);
		wait(delay);
		return (status & (SINGLEFRAMEDONE | STATICIMAGEDONE)) != 0;
	}

	public int status()
	{
		return status;
	}

	public PixelGrabber(ImageProducer ip, int x, int y, int w, int h, int[] pixels, int offset, int scanSize)
	{
		this.producer = ip;
		this.x = x;
		this.y = y;
		this.w = w;
		this.h = h;
		this.pixels = pixels;
		this.offset = offset;
		this.scanSize = scanSize;
	}

	public PixelGrabber(Image image, int x, int y, int w, int h, int[] pixels, int offset, int scanSize)
	{
		this(image.getSource(), x, y, w, h, pixels, offset, scanSize);
	}
}

