
// (C) 1997 Glynn Clements <glynn@sensei.co.uk> - Freely Redistributable

package java.awt.image;

public abstract class RGBImageFilter extends ImageFilter
{
	protected boolean canFilterIndexColorModel = false;
	protected ColorModel newmodel;
	protected ColorModel origmodel;

	public abstract int filterRGB(int x, int y, int rgb);

	public IndexColorModel filterIndexColorModel(IndexColorModel icm)
	{
		int n = icm.getMapSize();
		byte[] r = new byte[n];
		byte[] g = new byte[n];
		byte[] b = new byte[n];
		byte[] a = new byte[n];
		for (int i = 0; i < n; i++)
		{
			int pixel = icm.getRGB(i);
			pixel = filterRGB(-1, -1, pixel);
			r[i] = (byte) icm.getRed(pixel);
			g[i] = (byte) icm.getGreen(pixel);
			b[i] = (byte) icm.getBlue(pixel);
			a[i] = (byte) icm.getAlpha(pixel);
		}

		int bits = icm.getPixelSize();

		return new IndexColorModel(bits, n, r, g, b, a);
	}

	public void filterRGBPixels(int x, int y, int w, int h, int[] pixels, int off, int scansize)
	{
		int n0 = off + y * scansize + x;
		for (int j = 0; j < h; j++)
		{
			int n = n0;
			for (int i = 0; i < w; i++)
			{
				int rgb = pixels[n];
				pixels[n] = filterRGB(x + i, y + j, rgb);
				n++;
			}
			n0 += scansize;
		}

		super.setPixels(x, y, w, h, ColorModel.getRGBdefault(), pixels, off, scansize);
	}

	public void setColorModel(ColorModel model)
	{
		if ((model instanceof IndexColorModel) && canFilterIndexColorModel)
		{
			ColorModel newModel = filterIndexColorModel((IndexColorModel) model);
			substituteColorModel(model, newModel);
			super.setColorModel(newModel);
		}
		else
			super.setColorModel(ColorModel.getRGBdefault());
	}

	public void setPixels(int x, int y, int w, int h, ColorModel model, byte[] pixels, int off, int scansize)
	{
		if (model == origmodel)
		{
			super.setPixels(x, y, w, h, newmodel, pixels, off, scansize);
			return;
		}

		int[] rgbPixels = new int[pixels.length];

		int n0 = off + y * scansize + x;
		for (int j = 0; j < h; j++)
		{
			int n = n0;
			for (int i = 0; i < w; i++)
			{
				int src = pixels[n] & 0xFF;
				int rgb = model.getRGB(src);
				rgbPixels[n] = rgb;
				n++;
			}
			n0 += scansize;
		}

		filterRGBPixels(x, y, w, h, rgbPixels, off, scansize);
	}

	public void setPixels(int x, int y, int w, int h, ColorModel model, int[] pixels, int off, int scansize)
	{
		if (model == origmodel)
		{
			super.setPixels(x, y, w, h, newmodel, pixels, off, scansize);
			return;
		}

		int[] rgbPixels = new int[pixels.length];

		int n0 = off + y * scansize + x;
		for (int j = 0; j < h; j++)
		{
			int n = n0;
			for (int i = 0; i < w; i++)
			{
				int src = pixels[n];
				int rgb = model.getRGB(src);
				rgbPixels[n] = rgb;
				n++;
			}
			n0 += scansize;
		}

		filterRGBPixels(x, y, w, h, rgbPixels, off, scansize);
	}

	public void substituteColorModel(ColorModel oldcm, ColorModel newcm)
	{
		newmodel = newcm;
		origmodel = oldcm;
	}

	public RGBImageFilter()
	{
	}
}

