
// (C) 1997 Glynn Clements <glynn@sensei.co.uk> - Freely Redistributable

package java.io;

public class BufferedOutputStream extends FilterOutputStream
{ 
	protected byte[] buf;
	protected int count;

	public synchronized void flush() throws IOException
	{
		if (count > 0)
			super.write(buf, 0, count);
		count = 0;
		super.flush();
	}

	public synchronized void write(int b) throws IOException
	{
		if (count >= buf.length)
		{
			super.write(buf, 0, count);
			count = 0;
		}
		buf[count++] = (byte) b;
	}

	public synchronized void write(byte[] b, int off, int len) throws IOException
	{
		int left = buf.length - count;

		if (len <= left)
		{
			System.arraycopy(b, off, buf, count, len);
			count += len;
			return;
		}

		System.arraycopy(b, off, buf, count, left);
		off += left;
		len -= left;
		super.write(buf, 0, buf.length);

		int spare = len % buf.length;

		super.write(b, off, len - spare);
		off += len - spare;

		System.arraycopy(b, off, buf, 0, spare);

		count = spare;
	}

	public synchronized void write(byte[] b) throws IOException
	{
		write(b, 0, b.length);
	}

	public BufferedOutputStream(OutputStream out, int size)
	{
		super(out);
		buf = new byte[size];
	}

	public BufferedOutputStream(OutputStream out)
	{
		this(out, 512);
	}
}

