
// (C) 1997 Glynn Clements <glynn@sensei.co.uk> - Freely Redistributable

package java.io;

public class ByteArrayOutputStream extends OutputStream
{ 
	protected byte[] buf;
	protected int count;

	private void ensure(int length)
	{
		if (length <= buf.length)
			return;

		byte[] newbuff = new byte[buf.length * 2];
		System.arraycopy(buf, 0, newbuff, 0, count);
		buf = newbuff;
	}

	public synchronized void reset()
	{
		count = 0;
	}

	public synchronized int size()
	{
		return count;
	}

	public synchronized byte[] toByteArray()
	{
		try
		{
			// array clone no longer throws exception (in 1.1)
			if (false) throw new CloneNotSupportedException("Never thrown");
			return (byte[]) buf.clone();
		}
		catch (CloneNotSupportedException e)
		{
			return null;
		}
	}

	public synchronized String toString(int hibyte)
	{
		return new String(buf, hibyte, 0, count);
	}

	public synchronized String toString()
	{
		return toString(0);
	}

	public synchronized void write(int b)
	{
		ensure(count + 1);
		buf[count++] = (byte) b;
	}

	public synchronized void write(byte[] b, int off, int len)
	{
		ensure(count + len);
		System.arraycopy(b, off, buf, count, len);
		count += len;
	}

	public synchronized void writeTo(OutputStream out) throws IOException
	{
		out.write(buf, 0, count);
	}

	public ByteArrayOutputStream(int size)
	{
		buf = new byte[size];
	}

	public ByteArrayOutputStream()
	{
		this(32);
	}
}

