
// (C) 1997 Glynn Clements <glynn@sensei.co.uk> - Freely Redistributable

package java.io;

public class DataOutputStream extends FilterOutputStream implements DataOutput
{ 
	protected int written = 0;

	public void flush() throws IOException
	{
		super.flush();
	}

	public final int size()
	{
		return written;
	}

	public synchronized void write(int b) throws IOException
	{
		super.write(b);
		written++;
	}

	public synchronized void write(byte[] b, int off, int len) throws IOException
	{
		super.write(b, off, len);
		written += len;
	}

	public final void writeBoolean(boolean v) throws IOException
	{
		write(v ? 1 : 0);
	}

	public final void writeByte(int v) throws IOException
	{
		write(v);
	}

	public final synchronized void writeChar(int v) throws IOException
	{
		write((v>>>8) & 0xFF);
		write((v)     & 0xFF);
	}

	public final synchronized void writeShort(int v) throws IOException
	{
		write((v>>>8) & 0xFF);
		write((v)     & 0xFF);
	}

	public final synchronized void writeInt(int v) throws IOException
	{
		write((v >>> 24) & 0xFF);
		write((v >>> 16) & 0xFF);
		write((v >>> 8)  & 0xFF);
		write((v)        & 0xFF);
	}

	public final synchronized void writeLong(long v) throws IOException
	{
		writeInt((int)(v >>> 32));
		writeInt((int)v);
	}

	public final void writeFloat(float v) throws IOException
	{
		int n = Float.floatToIntBits(v);
		writeInt(n);
	}

	public final void writeDouble(double v) throws IOException
	{
		long n = Double.doubleToLongBits(v);
		writeLong(n);
	}

	public final void writeBytes(String s) throws IOException
	{
		int len = s.length();
		byte[] buff = new byte[len];
		s.getBytes(0, len, buff, 0);
		write(buff, 0, len);
	}

	public final synchronized void writeChars(String s) throws IOException
	{
		int len = s.length();
		char[] buff = new char[len];
		s.getChars(0, len, buff, 0);
		for (int i = 0; i < len; i++)
			writeChar((int) buff[i]);
	}

	private final void writeUTFByte(int c) throws IOException
	{
		write(0x80 + (c & 0x3F));
	}

	private final void writeUTFChar(int c) throws IOException
	{
		if (c <= 0x7F)
		{
			write(c);
		}
		else if (c <= 0x7FF)
		{
			write(0xC0 + (c >> 6));
			writeUTFByte(c);
		}
		else
		{
			write(0xE0 + (c >> 12));
			writeUTFByte(c >> 6);
			writeUTFByte(c);
		}
	}

	private final static int UTFBytesUsed(char[] buff, int len)
	{
		int byteCt = 0;

		for (int i = 0; i < len; i++)
		{
			int c = (int)(buff[i]);
			if (c <= 0x7F)
				byteCt++;
			else if (c <= 0x7FF)
				byteCt+=2;
			else
				byteCt+=3;
		}

		return byteCt;
	}

	public final synchronized void writeUTF(String s) throws IOException
	{
		int len = s.length();
		char[] buff = new char[len];
		s.getChars(0, len, buff, 0);

		// First figure out how many bytes the UTF string
		// uses, then write that info out.
		int byteCt = UTFBytesUsed(buff, len);
		writeShort(byteCt);
		
		// Now, write out the actual data bytes.
		for (int i = 0; i < len; i++)
			writeUTFChar((int) buff[i]);
	}

	public DataOutputStream(OutputStream out)
	{
		super(out);
	}
}

