
// (C) 1997 Glynn Clements <glynn@sensei.co.uk> - Freely Redistributable

package java.io;

import java.io.NativeIO;

public class FileInputStream extends InputStream
{
	private FileDescriptor fd;

	protected synchronized void finalize() throws IOException
	{
		if (fd != null)
			close();
	}

	public synchronized void close() throws IOException
	{
		if (fd == null)
			throw new IOException();
		NativeIO.close(fd);
		fd = null;
	}

	public FileDescriptor getFD() throws IOException
	{
		if (fd == null)
			throw new IOException();
		return fd;
	}

	public int available() throws IOException
	{
		if (fd == null)
			throw new IOException();
		return NativeIO.available(fd);
	}

	public int read() throws IOException
	{
		if (fd == null)
			throw new IOException();
		byte[] b = new byte[1];
		int n = NativeIO.read(fd, b, 0, 1);
		if (n < 0)
			return n;
		else
			return ((int)b[0]) & 0xFF;
	}

	public int read(byte[] b, int off, int len) throws IOException
	{
		if (fd == null)
			throw new IOException();
		return NativeIO.read(fd, b, off, len);
	}

	public int read(byte[] b) throws IOException
	{
		if (fd == null)
			throw new IOException();
		return NativeIO.read(fd, b, 0, b.length);
	}

	public long skip(long n) throws IOException
	{
		if (fd == null)
			throw new IOException();
		return (long) NativeIO.skip(fd, (int) n);
	}

	public FileInputStream(String name) throws FileNotFoundException
	{
		this.fd = new FileDescriptor();
		NativeIO.open_read(this.fd, name);
		if (!this.fd.valid())
			throw new FileNotFoundException();
	}

	public FileInputStream(File file) throws FileNotFoundException
	{
		this(file.getPath());
	}

	public FileInputStream(FileDescriptor fd)
	{
		this.fd = fd;
	}
}

