
// (C) 1997 Glynn Clements <glynn@sensei.co.uk> - Freely Redistributable

package java.io;

import java.io.NativeIO;

public class FileOutputStream extends OutputStream
{ 
	private FileDescriptor fd;

	protected synchronized void finalize() throws IOException
	{
		if (fd != null)
			close();
	}

	public synchronized void close() throws IOException
	{
		if (fd == null)
			throw new IOException();
		NativeIO.close(fd);
		fd = null;
	}

	public FileDescriptor getFD() throws IOException
	{
		if (fd == null)
			throw new IOException();
		return fd;
	}

	public void write(byte[] b, int off, int len) throws IOException
	{
		if (fd == null)
			throw new IOException();
		NativeIO.write(fd, b, off, len);
	}

	public void write(byte[] b) throws IOException
	{
		write(b, 0, b.length);
	}

	public void write(int b) throws IOException
	{
		byte[] buf = new byte[1];
		buf[0] = (byte) (b & 0xFF);
		write(buf);
	}

	public FileOutputStream(String name) throws IOException
	{
		this.fd = new FileDescriptor();
		NativeIO.open_write(this.fd, name);
		if (!this.fd.valid())
			throw new IOException();
	}

	public FileOutputStream(File file) throws IOException
	{
		this(file.getPath());
	}

	public FileOutputStream(FileDescriptor fd)
	{
		this.fd = fd;
	}
}

