
// (C) 1997 Glynn Clements <glynn@sensei.co.uk> - Freely Redistributable

package java.io;

public class LineNumberInputStream extends FilterInputStream
{ 
	private int line = 0;
	private int markline = -1;
	private int next = -2;

	private int read0() throws IOException
	{
		int c;

		if (next > -2)
		{
			c = next;
			next = -2;
		}
		else
			c = super.read();

		return c;
	}

	private void unput(int c) throws IOException
	{
		if (next > -2)
			throw new IOException();
		next = c;
	}

	public synchronized int available() throws IOException
	{
		return super.available() / 2 + (next >= 0 ? 1 : 0);
	}

	public synchronized void mark(int readlimit)
	{
		markline = line;
		super.mark(readlimit);
	}

	public boolean markSupported()
	{
		return super.markSupported();
	}

	public synchronized int read() throws IOException
	{
		int c = read0();

		if (c == '\n')
			line++;

		if (c != '\r')
			return c;

		c = read0();

		if (c == '\n')
			return '\n';

		unput(c);

		return '\r';
	}

	public synchronized int read(byte[] b, int off, int len) throws IOException
	{
		return super.read(b, off, len);
	}

	public synchronized void reset() throws IOException
	{
		if (markline < 0)
			throw new IOException();
		line = markline;
		super.reset();
	}

	public synchronized long skip(long n) throws IOException
	{
		for (int i = 0; i < n; i++)
		{
			int c = read();
			if (c < 0)
				return i;
		}

		return n;
	}

	public synchronized int getLineNumber()
	{
		return line;
	}

	public synchronized void setLineNumber(int n)
	{
		line = n;
	}

	public LineNumberInputStream(InputStream in)
	{
		super(in);
	}
}

