
// (C) 1997 Glynn Clements <glynn@sensei.co.uk> - Freely Redistributable

package java.io;

import java.io.FileDescriptor;
import java.io.IOException;
import java.io.FileNotFoundException;

/**
 * All of the Native methods used in the java.io package.
 *
 * @version Kore 0.0.3, June 1997
 * @author Glynn Clements <a href="mailto:glynn@sensei.co.uk">glynn@sensei.co.uk</a>
 */
class NativeIO
{
	public final static int SEEK_SET = 0;
	public final static int SEEK_CUR = 1;
	public final static int SEEK_END = 2;

	public static native int available(FileDescriptor fd) throws IOException;

	/**
	 * Read <code>len</code> bytes into array <code>b</code> at offset <code>off</code>
	 * from the given FileDescriptor.
	 * <p>
	 * <strong>Implementors:</strong> This function should never return zero.  Zero bytes
	 * read is an error, and it should return -1 in that case.
	 *
	 * @return Status of the read.  Either the number of bytes read (1 to <code>len</code>),
	 * 		or -1 if 0 bytes were read.
	 */
	public static native int read(FileDescriptor fd, byte[] b, int off, int len) throws IOException;
	public static native int write(FileDescriptor fd, byte[] b, int off, int len) throws IOException;

	/**
	 * Skip over <code>len</code> bytes of input in <code>fd</code>.
	 * This function may end up skipping fewer than <code>len</code>
	 * bytes.  In any case, the number of bytes skipped is returned.
	 * <p>
	 * This function is only ever called by FileInputStream.skip(n)
	 *
	 * @see FileInputStream.skip(long)
	 * @param fd The file descriptor to change the offset in.
	 * @param len the number of bytes to skip in fd.
	 * @return
	 */
	public static native int skip(FileDescriptor fd, int len) throws IOException;

	/**
	 * Close the given <code>FileDescriptor</code>.
	 * <p>
	 * <strong>Implementors:</strong> Should set the actual file descriptor
	 * associated with <code>fd</code> to -1.
	 *
	 * @param fd The <code>FileDescriptor</code> to close.
	 */
	public static native void close(FileDescriptor fd) throws IOException;

	/**
	 * open a file in read mode.
	 *
	 * @param fd A brand new <code>FileDescriptor</code> to fill in.
	 * @param name Name of the file to open.
	 */
	public static native void open_read(FileDescriptor fd, String name) throws FileNotFoundException;

	/**
	 * open a file in write mode.
	 *
	 * @param fd A brand new <code>FileDescriptor</code> to fill in.
	 * @param name Name of the file to open.
	 */
	public static native void open_write(FileDescriptor fd, String name) throws IOException;

	/**
	 * open a file in read-write mode.
	 *
	 * @param fd A brand new <code>FileDescriptor</code> to fill in.
	 * @param name Name of the file to open.
	 */
	public static native void open_read_write(FileDescriptor fd, String name) throws IOException;

	/**
	 * Return the current offset in the file descriptor <code>fd</code>.
	 *
	 * @param fd The <code>FileDescriptor</code> to get the offset from.
	 * @return Current offset in file fd.
	 */
	public static native int ftell(FileDescriptor fd) throws IOException;

	/**
	 * Seek <code>offset</code> bytes into <code>fd</code>;
	 * <code>whence</code> is one of the constants
	 * <code>NativeIO.SEEK_SET</code>,
	 * <code>NativeIO.SEEK_CUR</code>, or
	 * <code>NativeIO.SEEK_END</code>.
	 * <p>
	 * <strong>FIXME:</strong> Should offset be a long?
	 * <p>
	 * This function is only ever called from
	 * <code>PushbackFileInputStream</code>, and
	 * <code>RandomAccessFile</code>
	 *
	 * @see java.io.PushbackFileInputStream#unread(int)
	 * @see java.io.RandomAccessFile
	 * @param fd The <code>FileDescriptor</code> to seek in.
	 * @param offset The offset to seek to.
	 * @param whence How to interperet the seek, one of 
	 * 	<code>NativeIO.SEEK_SET</code>,
	 * 	<code>NativeIO.SEEK_CUR</code>, or
	 * 	<code>NativeIO.SEEK_END</code>.
	 * @return The absolute offset in the file after the seek.
	 */
	/* XXX should offset be a long? */
	public static native int fseek(FileDescriptor fd, int offset, int whence) throws IOException;

	/**
	 * Check the current permissions to see if the given <code>path</code>
	 * can be read.
	 *
	 * @param path Path to the file whose permissions need to be checked.
	 * @return a boolean indicating if the file can be read.
	 */
	public static native boolean canRead(String path);
	public static native boolean canWrite(String path);

	/**
	 * Return a boolean indicating if the given <code>path</code> exists.
	 * The path can be either relative or absolute.  The path may
	 * specifiy a directory or a real file.
	 *
	 * @param path Path to the file whose permissions need to be checked.
	 * @return a boolean indicating if the path exists.
	 */
	public static native boolean exists(String path);

	public static native boolean isDirectory(String path);

	/**
	 * Determine if given path points to a regular file.
	 * <p>
	 * This should only ever be called from
	 * <code>java.io.File.isFile()</code>
	 *
	 * @return A boolean indicating the regular file-itude of the
	 *		given <code>path</code>
	 * @param path A pathname.  Not necessarily absolute or anything.
	 * @see java.io.File#isFile()
	 */
	public static native boolean isFile(String path);
	public static native long lastModified(String path);
	public static native int fileSize(String path);
	public static native boolean delete(String path);
	public static native boolean rename(String from, String to);
	public static native boolean mkdir(String path);
	public static native String[] dirList(String path);
}

