
// (C) 1997 Glynn Clements <glynn@sensei.co.uk> - Freely Redistributable

package java.io;

public class PipedInputStream extends InputStream
{
	private final static int bufsize = 1024;

	private byte[] buff = new byte[bufsize];
	private int start = 0;
	private int end = 0;
	private int count = 0;
	private boolean closed = false;

	synchronized void write(int b) throws IOException
	{
		if (closed)
			throw new IOException();

		while (true)
		{
			if (count < bufsize)
			{
				buff[end] = (byte) b;
				end++;
				end %= buff.length;
				count++;
				notify();
				break;
			}
			try
			{
				wait();
			}
			catch (InterruptedException e)
			{
			}
		}
	}

	public synchronized void close() throws IOException
	{
		closed = true;
	}

	public synchronized void connect(PipedOutputStream src) throws IOException
	{
		src.connect(this);
	}

	public synchronized int read() throws IOException
	{
		while (true)
		{
			if (count > 0)
			{
				int c = buff[start];
				start++;
				start %= buff.length;
				count--;
				notify();
				return c;
			}

			if (closed)
				return -1;

			try
			{
				wait();
			}
			catch (InterruptedException e)
			{
			}
		}
	}

	public synchronized int read(byte[] b, int off, int len) throws IOException
	{
		for (int i = 0; i < len; i++)
		{
			int c = read();
			if (c < 0)
				return i == 0 ? -1 : i;
			b[off + i] = (byte) c;
		}

		return len;
	}

	public PipedInputStream(PipedOutputStream src) throws IOException
	{
		connect(src);
	}

	public PipedInputStream()
	{
	}
}

