
// (C) 1997 Glynn Clements <glynn@sensei.co.uk> - Freely Redistributable

package java.io;

import java.util.Vector;
import java.util.Enumeration;

public class SequenceInputStream extends InputStream
{
	private Vector streams = new Vector();
	private int index = 0;
	private InputStream in;

	private void nextStream()
	{
		if (index < streams.size())
			in = (InputStream) streams.elementAt(index++);
		else
			in = null;
	}

	public synchronized int read() throws IOException
	{
		while (true)
		{
			if (in == null)
				nextStream();
			if (in == null)
				return -1;

			int c = in.read();
			if (c >= 0)
				return c;

			in.close();
			in = null;
		}
	}

	public synchronized int read(byte[] buf, int pos, int len) throws IOException
	{
		int total = 0;

		while (len > 0)
		{
			if (in == null)
				nextStream();
			if (in == null)
				break;

			int n = in.read(buf, pos, len);
			if (n < 0)
			{
				in.close();
				in = null;
				continue;
			}

			pos += n;
			len -= n;
		}

		return (total == 0) ? -1 : total;
	}

	public synchronized void close() throws IOException
	{
		while (true)
		{
			in.close();
			nextStream();
			if (in == null)
				return;
		}
	}

	public SequenceInputStream(Enumeration e)
	{
		while (e.hasMoreElements())
			streams.addElement(e.nextElement());
	}

	public SequenceInputStream(InputStream s1, InputStream s2)
	{
		streams.addElement(s1);
		streams.addElement(s2);
	}
}

