
// (C) 1997 Glynn Clements <glynn@sensei.co.uk> - Freely Redistributable

package java.io;

public class StringBufferInputStream extends InputStream
{ 
	protected String buffer;
	protected int count;
	protected int pos;

	public synchronized int available()
	{
		return count - pos;
	}

	public synchronized void reset()
	{
		pos = 0;
	}

	public synchronized int read()
	{
		if (pos >= count)
			return -1;
		else
			return ((int)buffer.charAt(pos++)) & 0xFF;
	}

	public synchronized int read(byte[] b, int off, int len)
	{
		if (pos >= count)
			return -1;
		int bytes = Math.min(count - pos, len);
		buffer.getBytes(pos, pos + bytes, b, off);
		pos += bytes;
		return bytes;
	}

	public synchronized long skip(long n)
	{
		int target = (int) (pos + n);
		if (target > count)
		{
			int bytes = count - pos;
			pos = count;
			return bytes;
		}

		pos = (int) target;

		return n;
	}

	public StringBufferInputStream(String str)
	{
		buffer = str;
		pos = 0;
		count = str.length();
	}
}

