/*
 *  java.lang.Boolean
 *
 *  (c) 1997 George David Morrison
 *
 *  API version: 1.0.2
 *
 *  History:
 *  01JAN1997  George David Morrison
 *    Initial version
 */

package java.lang;

public final class Boolean
{
	private static final int TRUE_HASH = 1231;
	private static final int FALSE_HASH = 1237;
	private static final String TRUE_STRING = "true";
	private static final String FALSE_STRING = "false";
	private boolean value;

	public static final Boolean TRUE = new Boolean(true);
	public static final Boolean FALSE = new Boolean(false);

	public Boolean(boolean value)
	{
		this.value = value;
	}

	public Boolean(String s)
	{
		value = valueOf(s).booleanValue();
	}

	public String toString()
	{
		if (value)
			return TRUE_STRING;
		else
			return FALSE_STRING;
	}

	public boolean equals(Object obj)
	{
		if (obj != null && obj instanceof Boolean)
		{
			return (value == ((Boolean)obj).value);
		}
		return false;
	}

	public int hashCode()
	{
		if (value)
			return TRUE_HASH;
		else
			return FALSE_HASH;
	}

	public boolean booleanValue()
	{
		return value;
	}

	public static Boolean valueOf(String s)
	{
		if (s != null)
		{
			if (s.toLowerCase().equals(TRUE_STRING))
				return TRUE;
		}
		return FALSE;
	}

	public static boolean getBoolean(String name)
	{
		String propertyValue = System.getProperty(name);
		return valueOf(propertyValue).booleanValue();
	}
}

