/*
 *  java.lang.ClassLoader
 *
 *  (c) 1997 George David Morrison
 *
 *  API version: 1.0.2
 *
 *  History:
 *  01JAN1997  George David Morrison
 *    Initial version
 */

package java.lang;

import java.lang.NativeLang;

public abstract class ClassLoader
{
	protected abstract Class loadClass(String name, boolean resolve) throws ClassNotFoundException;

	protected final Class defineClass(byte[] data, int offset, int length)
	{
		if (data == null)
			throw new NullPointerException();
		if (offset < 0 || length < 0 || (offset + length) > data.length)
			throw new IndexOutOfBoundsException();

		Class c = NativeLang.defineClass(this, data, offset, length);
		if (c == null)
			throw new ClassFormatError();
		return c;
	}

	protected final void resolveClass(Class c)
	{
		if (c == null)
			throw new NullPointerException();
		NativeLang.resolveClass(c);
	}

	protected final Class findSystemClass(String name) throws ClassNotFoundException
	{
		if (name == null)
			throw new NullPointerException();
		Class c = NativeLang.findSystemClass(name);
		if (c == null)
			throw new ClassNotFoundException();
		return c;
	}

	protected ClassLoader()
	{
		SecurityManager securityManager = System.getSecurityManager();
		if (securityManager != null)
			securityManager.checkCreateClassLoader();
	}
}

