
// (C) 1997 Glynn Clements <glynn@sensei.co.uk> - Freely Redistributable

package java.lang;

/**
 * The Native methods used in the java.lang package.  There are other
 * native methods specified in java.lang.Object and java.lang.Class.
 *
 * @see java.lang.Object
 * @see java.lang.Class
 * @version Kore 0.0.3, June 1997
 * @author Glynn Clements <a href="mailto:glynn@sensei.co.uk">glynn@sensei.co.uk</a>
 */

class NativeLang
{
	/**
	 * Return the current time in milliseconds.
	 * <p>
	 * This method should only ever be called by
	 * <code>java.lang.System.currentTimeMillis</code>
	 *
	 * @return The number of milliseconds since <strong>The Epoch</strong>,
	 *		Jan 1, 1970.
	 * @see java.lang.System#currentTimeMillis()
	 */
	public static native long currentTimeMillis();

	public static native void exit(int status);
	public static native Process exec(String[] cmdarray, String[] envp);
	public static native boolean load(String filename);
	public static native boolean loadLibrary(String libname);
	public static native void gc();

	public static native long freeMemory();
	public static native long totalMemory();
	public static native void traceInstructions(boolean on);
	public static native void traceMethodCalls(boolean on);

	/**
	 * Return the name of the default properties file.
	 * <p>
	 * <strong>Implementors:</strong> It is recommened that this
	 * function return the equivalent of
	 * <pre>"$(HOME)/.kore"</pre>, or let the user inform the VM
	 * of a file to load via command line or via some other system
	 * dependent means.
	 *
	 * @return A fully-qualified string name of the default
	 * 	properties file.
	 */
	public static native String defaultPropertiesFile();
	
	/**
	 * Return the builtin VM properties in a large array of Strings.
	 * Even entries are properties, the following odd values
	 * are the value.  E.g., element zero is the property name and 
	 * element one is the associated value.
	 * <p>
	 * Do not include properties the user defined on the command
	 * line (if there is such a thing) in this list.  Command-line
	 * options should be parsed separately, see
	 * <code>getCommandLineProperties()</code>.
	 * <p>
	 * This function is only used for setting up the initial properties
	 * by <code>java.lang.System</code>.
	 * <p>
	 * This is not the most elegant interface, but it keeps the VM
	 * from having to understand the properties file (and hashtables.)
	 * <p>
	 * <strong>Implementors:</strong> This function should return a
	 * meaningful value for the following properties (at least.)
	 * Other standard or supported options should also be returned.
	 * (And added here if you think necessary.)
	 * <p>
	 * <blockquote>
	 * <strong>java.version:<\strong>	  Java version number
	 * <strong>java.vendor:<\strong>          Java vendor specific string
	 * <strong>java.vendor.url:<\strong>      Java vendor URL
	 * <strong>java.home:<\strong>            Java installation directory
	 * <strong>java.class.version:<\strong>   Java class version number
	 * <strong>java.class.path:<\strong>      Java classpath
	 * <strong>os.name:<\strong>              Operating System Name
	 * <strong>os.arch:<\strong>              Operating System Architecture
	 * <strong>os.version:<\strong>           Operating System Version
	 * <strong>file.separator:<\strong>       File separator ("/" on Unix)
	 * <strong>path.separator:<\strong>       Path separator (":" on Unix)
	 * <strong>line.separator:<\strong>       Line separator ("\n" on Unix)
	 * <strong>user.name:<\strong>            User account name
	 * <strong>user.home:<\strong>            User home directory
	 * <strong>user.dir:<\strong>             User's current working directory
	 * </blockquote>
	 *
	 * @see java.lang.System#initProperties()
	 * @see #getCommandLineProperties()
	 * @return An array of Strings, alternating property name and
	 * 	property values.  Must be an even-length array.
	 */
	public static native String[] getVMProperties();

	/**
	 * Return the properties the user defined on the VM command
	 * line, in an array of Strings.  Even entries are
	 * properties, the following odd values are the value.  E.g.,
	 * element zero is the property name and element one is the
	 * associated value.  Returns null if the user defined no
	 * properties on the VM command line.
	 * <p>
	 * This function is only used for setting up the initial properties
	 * by <code>java.lang.System</code>.
	 * <p>
	 * This is not the most elegant interface, but it keeps the VM
	 * from having to understand the properties file (and hashtables.)
	 * <p>
	 * 
	 */
	public static native String[] getCommandLineProperties();
	  
	/**
	 * Get a stack trace as an array of objects with a suitable
	 * toString() representation, or an array of chars
	 * (Throwable.fillInStackTrace() makes a special check for
	 * instances of char[] which doesn't have a proper toString()
	 * method.)
	 * <p>
	 * Only ever called by
	 * <code>java.lang.Throwable.fillInStackTrace()</code>
	 *
	 * @return An array of objects, that encode a stack trace.
	 * @see java.lang.Throwable#fillInStackTrace()
	 */
	public static native Object[] getStackTrace();

	/**
	 * Return an array of <code>Class</code> objects, one for the
	 * class of each method on the execution stack.
	 * <p>
	 * Only ever called by the
	 * <code>SecurityManager.getClassContext()</code>.
	 *
	 * @return An array of Class objects, one for the class of each  method
	 * 	on the execution stack.
	 */
	public static native Class[] getClassContext();

	/**
	 * Define a class object from an array of bytes.  The
	 * classLoader argument is special, in that a VM really only
	 * needs to know which class loader loaded what class (a class
	 * is defined by the pair (name, classloader)), and so it only
	 * needs the pointer to the class loader.
	 * <p>
	 * Only ever called by <code>java.lang.ClassLoader.defineClass()</code>
	 *
	 * @param classLoader the classloader making the request.
	 * @param data the array of bytes to make the class from.
	 * @param offset offset into the array.
	 * @param length length to look into the array.
	 */
	public static native Class defineClass(ClassLoader classLoader, byte[] data, int offset, int length);

	public static native void resolveClass(Class c);
	public static native Class findSystemClass(String name);

	/**
	 * Find current, active <code>java.lang.Thread</code>.
	 * 
	 * @return The current thread
	 * @see java.lang.Thread
	 */
	public static native Thread currentThread();

	/**
	 * Cause the current thread to sleep for the specified millis
	 * and nanos.  The thread may sleep for longer than the
	 * specified time.  Some implementations may not support
	 * sleeping at a finer granularity than millis, in which case
	 * the nanos argument is ignored.
	 * <p>
	 * This method should only ever be called by
	 * <code>java.lang.Thread.sleep()</code>
	 * <p>
	 * <strong>Implementors:</strong> This method must set the
	 * state of the current thread, thread-&gt;state, from
	 * RUNNING, to ASLEEP, to RUNNING. See
	 * <code>setThreadState()</code> and
	 * <code>java.lang.Thread</code> for more information on
	 * thread states.
	 *
	 * @param millis The number of millis to sleep for
	 * @param nanos The number of nanos to sleep for
	 * @see java.lang.Thread
	 * @see java.lang.Thread#sleep()
	 * @see java.lang.Thread#yield()
	 * @see java.lang.NativeLang#setThreadState(Thread, int)
	 */
	public static native void sleep(long millis, int nanos);

	/**
	 * Set the state of the current given thread.  States specified in the
	 * <code>java.lang.Thread</code> are:
	 * <blockquote><pre>
	 *    CREATED     = 0
	 *    RUNNING     = 1
	 *    SUSPENED    = 2
	 *    ASLEEP      = 3
	 *    INTERRUPTED = 4
	 *    STOPPED     = 5
	 *    DESTROYED   = 6
	 *    WAITING     = 7
	 * </pre></blockquote>
	 * <p>
	 * This function should should only ever be called from
	 * <code>java.lang.Thread</code> methods.
	 * <p>
	 * <strong>Implementors:</strong> This function should set the
	 * state of the Thread, and
	 * take whatever VM-specific action is necessary to put the underlying
	 * thread into that state.
	 * Note that the VM is not (directly) notified when a thread is
	 * created.
	 *
	 * @see java.lang.Thread
	 * @see java.lang.Thread#setState(int)
	 * @param thread The thread object to set the state of.
	 * @param newState The new state (must be one of the above) to set.
	 */
	public static native void setThreadState(Thread thread, int newState);


	/**
	 * Set the priority of the given thread.
	 * <p>
	 * This function should only ever be called from
	 * <code>java.lang.Thread.setPriority()</code>.
	 * <p>
	 * <strong>Implementors:</strong> 
	 * Checks on the validity of the priority are done in the
	 * caller, so the native method can just update whatever state
	 * its scheduler needs update directly.
	 *
	 * @see java.lang.Thread
	 * @see java.lang.Thread#setPriority(int)
	 * @param thread The thread object to set the priority of.
	 * @param priority The new priority of the thread.
	 */
	public static native void setThreadPriority(Thread thread, int priority);
		

	/**
	 * Yield the current thread.
	 * <p>
	 * Java yield semantics dictate that the highest-priority thread be
	 * made runnable after the current thread yeilds.  The thread picked
	 * <em>may be the current one</em> if it is the highest-priority thread
	 * available to run.
	 * <p>
	 * This function should only ever be called by 
	 * <code>java.lang.Thread.yield</code>.
	 *
	 * @see java.lang.Thread
	 * @see java.lang.Thread#yield()
	 */
	public static native void yield();

	/**
	 * Stop the given thread in its tracks.  Throw <code>error</code>
	 * (which is guaranteed to be non-null) in the target thread.
	 * <p>
	 * This method should only ever be called by
	 * <code>java.lang.Thread.stop(Throwable error)</code>.
	 * <p>
	 * <strong>Implementors:</strong> This must update the
	 * target thread's <code>state</code>
	 * to STOPPED.  It would simply be replaced with
	 * <code>setThreadState(this, STOPPED)</code> but can't be
	 * because of the <code>error</code> parameter.
	 *
	 * @see java.lang.Thread#stop(Throwable)
	 * @see java.lang.Thread#setState(int)
	 * @see java.lang.NativeLang#setThreadState(Thread, int)
	 * @param thread The thread to stop
	 * @param error The error message to throw in the target.  Always non-null.
	 */
	public static native void stopThread(Thread thread, Throwable error);
	public static native int countStackFrames(Thread thread);
}

