// java.lang.Runtime
// (c) 1997 Peter Nagy
// API version: 1.0.2
// History:
//  1997-01-28	Initial version	Peter Nagy
//  1997-02-25	modified	Glynn Clements <glynn@sensei.co.uk>
//  1997-12-02  Glynn Clements <glynn@sensei.co.uk>
//	inserted calls to freeMemory, totalMemory, traceInstructions,
//	and traceMethodCalls

package java.lang;

import java.lang.NativeLang;

import java.io.InputStream;
import java.io.OutputStream;
import java.io.IOException;

import java.util.StringTokenizer;

public class Runtime
{
	private static Runtime runtime = new Runtime();

	public static Runtime getRuntime()
	{
		return runtime;
	}

	public void exit(int status)
	{
		SecurityManager s = System.getSecurityManager();
		if (s != null)
			s.checkExit(status);

		NativeLang.exit(status);
	}

	public Process exec(String[] cmdarray, String[] envp) throws IOException
	{
		String command = cmdarray[0];

		SecurityManager s = System.getSecurityManager();
		if (s != null)
			s.checkExec(command);

		if (envp == null)
			envp = new String[0];

		Process p = NativeLang.exec(cmdarray, envp);
		if (p == null)
			throw new IOException("Exec error");

		return p;
	}

	public Process exec(String[] cmdarray) throws IOException
	{
		return exec(cmdarray, null);
	}

	public Process exec(String command, String[] envp) throws IOException
	{
		StringTokenizer tokenizer = new StringTokenizer(command);
		String[] cmdarray = new String[tokenizer.countTokens()];
		for (int n = 0; tokenizer.hasMoreTokens(); n++)
			cmdarray[n] = tokenizer.nextToken();
		return exec(cmdarray, envp);
	}

	public Process exec(String command) throws IOException
	{
		return exec(command, null);
	}

	public void load(String filename)
	{
		SecurityManager s = System.getSecurityManager();
		if (s != null)
			s.checkLink(filename);

		if (!NativeLang.load(filename))
			throw new UnsatisfiedLinkError(filename);
	}
  
	public void loadLibrary(String libname)
	{
		SecurityManager s = System.getSecurityManager();
		if (s != null)
			s.checkLink(libname);

		if (!NativeLang.loadLibrary(libname))
			throw new UnsatisfiedLinkError(libname);
	}

	public long freeMemory()
	{
		return NativeLang.freeMemory();
	}

	public long totalMemory()
	{
		return NativeLang.totalMemory();
	}

	public void gc()
	{
		NativeLang.gc();
	}

	public void runFinalization()
	{
		NativeLang.gc();
	}

	public void traceInstructions(boolean on)
	{
		NativeLang.traceInstructions(on);
	}

	public void traceMethodCalls(boolean on)
	{
		NativeLang.traceMethodCalls(on);
	}

	public InputStream getLocalizedInputStream(InputStream in)
	{
		return in;	// TODO: implement this properly
	}

	public OutputStream getLocalizedOutputStream(OutputStream out)
	{
		return out;	// TODO: implement this properly
	}

	private Runtime()
	{
	}
}

