// java.lang.SecurityManager
// (c) 1997 Peter Nagy
// API version: 1.0.2
// History:
//  1997-02-01 Initial version	Peter Nagy
//  1997-02-25 modified		Glynn Clements <glynn@sensei.co.uk>

package java.lang;

import java.lang.NativeLang;

import java.io.FileDescriptor;

public abstract class SecurityManager
{
	protected boolean inCheck = false;

	// helper functions - protected

	protected Class[] getClassContext()
	{
		return NativeLang.getClassContext();
	}

	protected int classDepth(String name)
	{
		Class[] stack = getClassContext();
		for (int i = 0; i < stack.length; i++)
			if (stack[i].getName().equals(name))
				return i;
		return -1;
	}

	protected boolean inClass(String name)
	{
		return classDepth(name) >= 0;
	}

	protected int classLoaderDepth()
	{
		Class[] stack = getClassContext();
		for (int i = 0; i < stack.length; i++)
			if (stack[i].getClassLoader() != null)
				return i;
		return -1;
	}

	protected ClassLoader currentClassLoader()
	{
		Class[] stack = getClassContext();
		for (int i = 0; i < stack.length; i++)
		{
			ClassLoader loader = stack[i].getClassLoader();
			if (loader != null)
				return loader;
		}
		return null;
	}

	protected boolean inClassLoader()
	{
		return currentClassLoader() != null;
	}

	// miscellaneous functions

	public boolean getInCheck()
	{
		return inCheck;
	}

	public Object getSecurityContext()
	{
		return null;
	}

	// now for the paranoic default - subclass it!

	public boolean checkTopLevelWindow(Object window)
	{
		return false;
	}

	public void checkRead(String file, Object context)
	{
		throw new SecurityException();
	}

	public void checkRead(FileDescriptor fd)
	{
		throw new SecurityException();
	}

	public void checkRead(String file)
	{
		throw new SecurityException();
	}

	public void checkWrite(FileDescriptor fd)
	{
		throw new SecurityException();
	}

	public void checkWrite(String file)
	{
		throw new SecurityException();
	}

	public void checkListen(int port)
	{
		throw new SecurityException();
	}

	public void checkAccept(String host, int port)
	{
		throw new SecurityException();
	}

	public void checkConnect(String host, int port)
	{
		throw new SecurityException();
	}

	public void checkConnect(String host, int port, Object context)
	{
		throw new SecurityException();
	}

	public void checkDelete(String file)
	{
		throw new SecurityException();
	}

	public void checkExec(String cmd)
	{
		throw new SecurityException();
	}

	public void checkAccess(Thread t)
	{
		throw new SecurityException();
	}

	public void checkAccess(ThreadGroup g)
	{
		throw new SecurityException();
	}

	public void checkCreateClassLoader()
	{
		throw new SecurityException();
	}

	public void checkExit(int status)
	{
		throw new SecurityException();
	}

	public void checkLink(String lib)
	{
		throw new SecurityException();
	}

	public void checkPackageAccess(String pkg)
	{
		throw new SecurityException();
	}

	public void checkPackageDefinition(String pkg)
	{
		throw new SecurityException();
	}

	public void checkPropertyAccess(String key)
	{
		throw new SecurityException();
	}

	public void checkPropertyAccess(String key,String def)
	{
		throw new SecurityException();
	}

	public void checkPropertiesAccess()
	{
		throw new SecurityException();
	}

	public void checkSetFactory()
	{
		throw new SecurityException();
	}

	protected SecurityManager()
	{
		if (System.getSecurityManager() != null)
			throw new SecurityException("SecurityManager already installed");
	}
}

