
// (C) 1997 Glynn Clements <glynn@sensei.co.uk> - Freely Redistributable

package java.net;

import java.net.NativeNet;

public final class InetAddress
{
	final static InetAddress INADDR_ANY = new InetAddress(-1);

	public static InetAddress[] getAllByName(String host)
	{
		byte[][] src = NativeNet.getAllByName(host);
		InetAddress[] dst = new InetAddress[src.length];
		for (int i = 0; i < src.length; i++)
			dst[i] = new InetAddress(src[i]);
		return dst;
	}

	public static InetAddress getLocalHost()
	{
		return new InetAddress(NativeNet.getLocalHost());
	}

	public static InetAddress getByName(String host)
	{
		return new InetAddress(NativeNet.getAllByName(host)[0]);
	}

	private static int i(byte b)
	{
		return b & 0xFF;
	}

	private static byte b(int i)
	{
		return (byte) i;
	}

	String hostname = null;
	byte[] addr;

	public boolean equals(Object obj)
	{
		if (!(obj instanceof InetAddress))
			return false;
		
		InetAddress a = (InetAddress) obj;

		if (a.addr.length != addr.length)
			return false;

		for (int i = 0; i < addr.length; i++)
			if (a.addr[i] != addr[i])
				return false;

		return true;
	}

	public byte[] getAddress()
	{
		return addr;
	}

	public String getHostName()
	{
		if (hostname == null)
			hostname = NativeNet.getHostName(addr);
		return hostname;
	}

	public int hashCode()
	{
		byte[] v = new byte[4];

		for (int i = 0; i < addr.length; i++)
			v[i & 3] += addr[i];

		return	(i(v[0]) <<  0) +
			(i(v[1]) <<  8) +
			(i(v[2]) << 16) +
			(i(v[3]) << 24);
	}

	public String toString()
	{
		return	"" + i(addr[0]) +
			"." + i(addr[1]) +
			"." + i(addr[2]) +
			"." + i(addr[3]);
	}

	InetAddress(byte[] addr)
	{
		this.addr = addr;
	}

	InetAddress(int addr)
	{
		byte[] b = new byte[4];
		b[0] = b(addr);	addr >>>= 8;
		b[1] = b(addr);	addr >>>= 8;
		b[2] = b(addr);	addr >>>= 8;
		b[3] = b(addr);

		this.addr = b;
	}
}

