
// (C) 1997 Glynn Clements <glynn@sensei.co.uk> - Freely Redistributable

package java.net;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.FileDescriptor;

public final class Socket
{ 
	private static SocketImplFactory factory;

	public static void setSocketImplFactory(SocketImplFactory fac) throws IOException
	{
		if (factory != null)
			throw new SocketException();
		factory = fac;
	}

	SocketImpl impl;

	public void close() throws IOException
	{
		impl.close();
	}

	public InetAddress getInetAddress()
	{
		return impl.getInetAddress();
	}

	public int getPort()
	{
		return impl.getPort();
	}

	public int getLocalPort()
	{
		return impl.getLocalPort();
	}

	public InputStream getInputStream() throws IOException
	{
		return impl.getInputStream();
	}

	public OutputStream getOutputStream() throws IOException
	{
		return impl.getOutputStream();
	}

	public String toString()
	{
		return super.toString();
	}

	public Socket(InetAddress address, int port, boolean stream) throws IOException
	{
		if (factory != null)
			impl = factory.createSocketImpl();
		else
			impl = new DefaultSocketImpl();

		impl.create(stream);
		impl.connect(address, port);
	}

	public Socket(InetAddress address, int port) throws IOException
	{
		this(address, port, true);
	}

	public Socket(String host, int port, boolean stream) throws IOException
	{
		this(InetAddress.getByName(host), port, stream);
	}

	public Socket(String host, int port) throws IOException
	{
		this(host, port, true);
	}

	Socket() throws IOException
	{
	}
}

