
// (C) 1997 Glynn Clements <glynn@sensei.co.uk> - Freely Redistributable

package java.net;

import java.io.IOException;
import java.io.InputStream;
import java.util.Hashtable;

public final class URL
{ 
	private static URLStreamHandlerFactory factory;

	private static Hashtable handlers = new Hashtable();

	private static Class cObject;
	private static Class cURLStreamHandler;

	private static boolean isURLStreamHandler(Class a)
	{
		try
		{
			if (cObject == null)
				cObject = Class.forName("java.lang.Object");
			if (cURLStreamHandler == null)
				cURLStreamHandler = Class.forName("java.net.URLStreamHandler");
		}
		catch (ClassNotFoundException e)
		{
		}

		while (true)
		{
			if (a.equals(cObject))
				return false;
			if (a.equals(cURLStreamHandler))
				return true;
			a = a.getSuperclass();
		}
	}

	private static URLStreamHandler makeHandler(String protocol)
	{
		URLStreamHandler handler = null;

		if (factory != null)
			handler = factory.createURLStreamHandler(protocol);

		if (handler != null)
			return handler;

		String pkgs = System.getProperty("java.handler.protol.pkgs");
		for (int i = 0; ; )
		{
			int end = pkgs.indexOf('|', i);
			String pkg = (end < 0)
				? pkgs.substring(i, end)
				: pkgs.substring(i);

			String name = pkg + "." + protocol + "." + "Handler";

			try
			{
				Class clazz = Class.forName(name);
				if (!isURLStreamHandler(clazz))
					continue;
				handler = (URLStreamHandler) clazz.newInstance();
				break;
			}
			catch (ClassNotFoundException e)
			{
			}
			catch (InstantiationException e)
			{
			}
			catch (IllegalAccessException e)
			{
			}

			if (end < 0)
				break;
			i = end + 1;
		}

		if (handler != null)
			return handler;

		try
		{
			String name = "sun.net.www.protocol." + protocol + ".Handler";
			Class clazz = Class.forName(name);
			if (!isURLStreamHandler(clazz))
				return null;
			handler = (URLStreamHandler) clazz.newInstance();
		}
		catch (ClassNotFoundException e)
		{
			return null;
		}
		catch (InstantiationException e)
		{
			return null;
		}
		catch (IllegalAccessException e)
		{
			return null;
		}

		return handler;
	}

	private static URLStreamHandler getHandler(String protocol)
	{
		URLStreamHandler handler = (URLStreamHandler) handlers.get(protocol);
		if (handler != null)
			return handler;

		handler = makeHandler(protocol);
		if (handler != null)
			handlers.put(protocol, handler);

		return handler;
	}

	public static void setURLStreamHandlerFactory(URLStreamHandlerFactory fac)
	{
		if (factory != null)
			throw new Error();
		factory = fac;
	}

	private URLConnection connection;
	private URLStreamHandler handler;

	String protocol;
	String host;
	int port = -1;
	String file;
	String ref;

	public boolean sameFile(URL u)
	{
		return	protocol.equals(u.protocol) &&
			host.equals(u.host) &&
			port == u.port &&
			file.equals(u.file);
	}

	public boolean equals(Object obj)
	{
		if (!(obj instanceof URL))
			return false;
		URL u = (URL) obj;

		return sameFile(u) && ref.equals(u.ref);
	}

	public int hashCode()
	{
		return	protocol.hashCode() ^
			host.hashCode() ^
			port ^
			file.hashCode();
	}

	public String toExternalForm()
	{
		return handler.toExternalForm(this);
	}

	public String toString()
	{
		return toExternalForm();
	}

	public final Object getContent() throws IOException
	{
		return openConnection().getContent();
	}

	public String getFile()
	{
		return file;
	}

	public String getHost()
	{
		return host;
	}

	public int getPort()
	{
		return port;
	}

	public String getProtocol()
	{
		return protocol;
	}

	public String getRef()
	{
		return ref;
	}

	public URLConnection openConnection() throws IOException
	{
		if (handler == null)
			handler = getHandler(protocol);
		if (connection == null)
			connection = handler.openConnection(this);
		return connection;
	}

	public final InputStream openStream() throws IOException
	{
		return openConnection().getInputStream();
	}

	public URL(String protocol, String host, int port, String file) throws MalformedURLException
	{
		this.protocol = protocol;
		this.host = host;
		this.port = port;
		this.file = file;

		handler = getHandler(protocol);
	}

	public URL(String protocol, String host, String file) throws MalformedURLException
	{
		this(protocol, host, -1, file);
	}

	public URL(URL context, String spec) throws MalformedURLException
	{
		int colon = spec.indexOf(':');
		int slash = spec.indexOf('/');
		int pound = spec.indexOf('#');
		int limit = (pound > 0) ? pound : spec.length();

		if (colon > 0 && (slash < 0 || slash > colon))
		{
			protocol = spec.substring(0, colon);
			
			if (context != null && context.protocol.equals(protocol))
			{
				host = context.host;
				port = context.port;
				file = context.file;
			}
		}
		else
		{
			if (context == null)
				throw new MalformedURLException();
			protocol = context.protocol;
		}

		handler = getHandler(protocol);
		handler.parseURL(this, spec, colon + 1, limit);
	}

	public URL(String spec) throws MalformedURLException
	{
		this(null, spec);
	}
}

