
// (C) 1996 Glynn Clements <glynn@sensei.co.uk> - Freely Redistributable

package java.util;

// This algorithm is dictated by The Java Language Specification, p646-

public class Random extends java.lang.Object
{ 
	// Fields

	protected long seed;
	protected double nextNextGaussian;
	protected boolean haveNextNextGaussian = false;
 
	// Protected Methods

	protected int next(int bits)
	{
		seed = (seed * 0x5DEECE66DL + 0xBL) & ((1L << 48) - 1);
		return (int)(seed >>> (48 - bits));
	}

	// Public Methods 

	public int nextInt()
	{
		return next(32);
	}

	public long nextLong()
	{
		return ((long) next(32) << 32) + next(32);
	}

	public float nextFloat()
	{
		return next(24) / (float)(1 << 24);
	}

	public double nextDouble()
	{
		return (((long) next(26) << 27) + next(27)) / (double)(1L << 53);
	}

	synchronized public double nextGaussian()
	{
		if (haveNextNextGaussian)
		{
			haveNextNextGaussian = false;
			return nextNextGaussian;
		}
		else
		{
			double v1, v2, s;

			do {
				v1 = 2 * nextDouble() - 1;
				v2 = 2 * nextDouble() - 1;
				s = v1 * v1 + v2 * v2;
			} while (s >= 1);

			double norm = Math.sqrt(-2 * Math.log(s)/s);

			nextNextGaussian = v2 * norm;
			haveNextNextGaussian = true;

			return v1 * norm;
		}
	}

	synchronized public void setSeed(long seed)
	{
		this.seed = (seed ^ 0x5DEECE66DL) & ((1L << 48) - 1);
		haveNextNextGaussian = false;
	}


	// Constructors 

	public Random()
	{
		this(System.currentTimeMillis());
	}

	public Random(long seed)
	{
		setSeed(seed);
	}
}

