
// (C) 1997 Glynn Clements <glynn@sensei.co.uk> - Freely Redistributable

package java.util;

public class StringTokenizer implements Enumeration
{
	private String str;
	private String delim;
	private boolean returnTokens;

	private String next;
	private int index = 0;

	private boolean isDelim(char c)
	{
		return delim.indexOf(c) >= 0;
	}

	private char nextChar()
	{
		return str.charAt(index++);
	}

	private void unreadChar()
	{
		index--;
	}

	private boolean atEOF()
	{
		return index >= str.length();
	}

	private String readToken()
	{
		if (atEOF())
			return null;

		StringBuffer s = new StringBuffer();

		char c = nextChar();

		if (returnTokens)
		{
			if (isDelim(c))
			{
				s.append(c);
				return s.toString();
			}
		}
		else
		{
			while (isDelim(c))
				if (atEOF())
					return null;
				else
					c = nextChar();
		}

		while (true)
		{
			if (isDelim(c))
			{
				unreadChar();
				return s.toString();
			}
			s.append(c);
			if (atEOF())
				return s.toString();
			c = nextChar();
		}
	}

	private void doToken()
	{
		next = readToken();
	}

	public synchronized int countTokens()
	{
		int save = index;

		int n;
		for (n = 0; next != null; n++)
			doToken();

		index = save;

		return n;
	}

	public boolean hasMoreTokens()
	{
		return next != null;
	}

	public boolean hasMoreElements()
	{
		return hasMoreTokens();
	}

	public synchronized String nextToken() throws NoSuchElementException
	{
		if (next == null)
			throw new NoSuchElementException();
		String token = next;
		doToken();
		return token;
	}

	public synchronized String nextToken(String delim) throws NoSuchElementException
	{
		this.delim = delim;
		return nextToken();
	}

	public Object nextElement()
	{
		return nextToken();
	}


	public StringTokenizer(String str, String delim, boolean returnTokens)
	{
		this.str = str;
		this.delim = delim;
		this.returnTokens = returnTokens;

		while (index < str.length() && delim.indexOf(str.charAt(index)) >= 0)
			index++;
	}

	public StringTokenizer(String str, String delim)
	{
		this(str, delim, false);
	}

	public StringTokenizer(String str)
	{
		this(str, " \t\n\r", false);
	}
}

