// (C) 1997 Glynn Clements <glynn@sensei.co.uk> - Freely Redistributable

package kore.util;

/**
 * A simple utility class for debugging the rest of the
 * Kore library.  This class should be made private or
 * removed completely from a "shippable" version of
 * Kore.
 *
 * Some rudimentary support for controlling debugging
 * output is provided through system properties, see
 * java.lang.System for details.
 *
 * @see java.lang.System
 */
public class NativeDebug
{
	/** 
	 * The following constants define various debugging
	 * options that can be selected.  Each debugging
	 * operation is defines a set of options that
	 * it is associated with, then the global mask is
	 * set and only those debugging operations that fall
	 * into the current mask are actually executed.
	 */

	public static final int HASHTABLE  = (1 << 1);
	public static final int PROPERTIES = (1 << 2);
	public static final int STRING     = (1 << 3);
	public static final int FILE       = (1 << 4);

	private static int debugMask = 0;

	public static boolean maskContains(int mask)
	{
		return ((mask & debugMask) != 0);
	}

	public static void setDebugMask(String masks)
	{
		// XXX support more than one mask (separated by ,'s)
		
		debugMask = 0;

		if (masks == null)
			return;

		if (masks.equalsIgnoreCase("list"))
		{
			printStrln0("Valid debug flags:");
			printStrln0("hashtable (controls java.util.hashtable debugging)");
			printStrln0("properties (controls java.util.properties debugging)");
			printStrln0("string (controls java.lang.String debugging)");
			printStrln0("file (controls java.io.File debugging)");

			java.lang.System.exit(11);
		}

		if (masks.equalsIgnoreCase("hashtable"))
			debugMask |= HASHTABLE;
		else if (masks.equalsIgnoreCase("properties"))
			debugMask |= PROPERTIES;
		else if (masks.equalsIgnoreCase("file"))
			debugMask |= FILE;
		else if (masks.equalsIgnoreCase("string"))
			debugMask |= STRING;
	}

	public static native void printStr0(String str);
	public static native void printStrln0(String str);
	public static native void printInt0(int theInt);
	public static native void here0();
	public static native void printTrace0();

	public static void printStr(int mask, String str)
	{
		if ((mask & debugMask) != 0)
			printStr0(str);
	}
	
	public static void printStrln(int mask, String str)
	{
		if ((mask & debugMask) != 0)
			printStrln0(str);
	}
	
	public static void printInt(int mask, int theInt)
	{
		if ((mask & debugMask) != 0)
			printInt0(theInt);
	}
	
	public static void here(int mask)
	{
		if ((mask & debugMask) != 0)
			here0();
	}
	
	public static void printTrace(int mask)
	{
		if ((mask & debugMask) != 0)
			printTrace0();
	}
	
}

