/*
 * Main.java
 *
 * Test Java File I/O primitives.
 */
import java.io.FileOutputStream;
import java.io.FileInputStream;

class FileIO
{
	public static void main(String[] args)
	{
		createFile("testoutput.bin");
		readFile("testoutput.bin");
		out("Success");
	}

	public static void out(String arg)
	{
		System.out.println(arg);
	}
	
	public static void createFile(String name)
	{
		try
		{
			FileOutputStream fos = new FileOutputStream(name);
			int i;
			
			for (i = 0; i < (1024 * 10); i++)
			{
				fos.write(i%256);
			}
			fos.close();
		}
		catch (java.io.IOException ioe)
		{
			out("createFile() failure: " +ioe);
			System.exit(11);
		}

		out("createFile() succeeded");
	}
	
	public static void readFile(String name)
	{
		try
		{
			FileInputStream fis = new FileInputStream(name);
			int i;
			
			for (i = 0; i < (1024 * 10); i++)
			{
				int b = fis.read();
				int expected = i%256;
				if (b != expected)
				{
					out("readFile(), test 1, failure: expected " 
					    +expected+ ", but got " +b);
					System.exit(11);
				}
			}
			fis.close();
		}
		catch (java.io.IOException ioe)
		{
			out("readFile(), test 1, failure: " +ioe);
			System.exit(11);
		}

		out("Passed read test 1");

		try
		{
			FileInputStream fis = new FileInputStream(name);
			int chunkSize = 512;
			int i,j;
			
			for (i = 0; i < (1024 * 10); i += chunkSize)
			{
				byte[] chunk = new byte[chunkSize];
				fis.read(chunk, 0, chunkSize);

				for (j = i; j < i + chunkSize; j++)
				{
					int expected = j%256;
					if (chunk[j-i] != (byte)expected)
					{
						out("readFile(), test 2, failure: expected " 
						    +expected+ ", but got " +chunk[j-i]);
						System.exit(11);
					}
				}
			}
			fis.close();
		}
		catch (java.io.IOException ioe)
		{
			out("readFile(), test 2, failure: " +ioe);
			System.exit(11);
		}

		out("Passed read test 2");
	}
}
