/*
 * Test to see if finalizer methods are
 * called.
 */

class Finalizer 
{
	static final int maxAllocs = 1024;
	static final int printEach = 64;
	static final int wastage   = (16 * 1024);

	class ChunkNoFinalizer
	{
		public int wasteOfSpace[];
		public int size;
		private int id_;
		
		public ChunkNoFinalizer(int id, int size)
		{
			int i;
			
			id_ = id;
			
			wasteOfSpace = new int[size];
			
			for (i = 0; i < size; i++)
			{
				wasteOfSpace[i] = i;
			}
		}
	}
	
	class ChunkFinalizer
	{
		public int wasteOfSpace[];
		public int size;
		private int id_;
		
		public ChunkFinalizer(int id, int size)
		{
			int i;
			
			id_ = id;
			
			wasteOfSpace = new int[size];
			
			for (i = 0; i < size; i++)
			{
				wasteOfSpace[i] = i;
			}
		}
		
		protected void finalize()
		{
			int i;
			
			if ((id_%printEach)==0)
				System.out.println("Finalizing " +id_);
			
			for (i=0; i < size; i++)
			{
				wasteOfSpace[i] = -1;
			}
		}
	}
	
	public static void main (String args[])
	{
		Finalizer f = new Finalizer();
		
		System.out.println("Allocating " +maxAllocs+ " chunks of " +wastage+ " ints (approx " +((wastage*4)+32)+ " bytes).");

		System.out.println("Allocating the no-finalizer variety.");
		f.goNoFinalizer();

		System.gc();

		System.out.println("Allocating the finalizer variety.");
		f.goFinalizer();

		System.out.println("SUCCESS");
	}
			

	public void goNoFinalizer()
	{
		int i;
		ChunkNoFinalizer w;

		for (i = 0; i < maxAllocs; i++)
		{
			w = new ChunkNoFinalizer(i, wastage);
			if ((i%printEach)==0)
				System.out.println("Allocated chunk " +i);
		}
	}

	public void goFinalizer()
	{
		int i;
		ChunkFinalizer w;

		for (i = 0; i < maxAllocs; i++)
		{
			w = new ChunkFinalizer(i, wastage);
			if ((i%printEach)==0)
				System.out.println("Allocated chunk " +i);
		}
	}
}
