import java.lang.ClassLoader;
import java.util.Hashtable;
import java.io.*;

class RootClassLoader extends ClassLoader
{
    private String _name;
    private Hashtable _classes;
    
    public RootClassLoader(String name)
    {
	_name = name;
	_classes = new Hashtable();
    }

    public Class loadClass(String name)
	throws ClassNotFoundException
    {
	return loadClass(name, false);
    }

    public Class loadClass(String name, boolean resolve)
	throws ClassNotFoundException
    {
	Class newClass; // Class file for class 'name'

	System.out.println(_name+ ": loadClass(" +name+ ", " +resolve+ ")");

	// if (name.equal("java.lang.System"))  ...

	// If its in the hashtable, return it; resolve if necessary
	newClass = (Class)_classes.get(name);
	if (newClass != null) {
	    System.out.println(_name+ ": hit in hashtab.");
	    if (resolve)
		resolveClass(newClass);
	    return newClass;
	}

	byte buf[] = null; // buffer to read file into
	
	try {
	    buf = bytesForClass(name); // get the .class file into byte array 
	} catch (IOException iox) {
	    // ie, name+".class" doesn't exist.
	    // System.out.println(_name+ ": IO Exception loading " +name);
	}
	
	if (buf == null) {
	    // If My loader couldn't find it, let the system deal. 
	    System.out.println(_name+ ": loading " +name+ " via system loader");
	    newClass = findSystemClass(name);
	} else {
	    System.out.println(_name+ ": call defineClass()");
	    // Define class, this is where kaffe differs from the world.
	    newClass = defineClass(buf, 0, buf.length);
	    System.out.println(_name+ ": done defineClass()");
	}
	    
	// add it to the list of found classes.
	_classes.put(name, newClass);

	if (resolve) {
	    System.out.println(_name+ ": resolving " +name);
	    resolveClass(newClass);
	}
	
	System.out.println(_name+ ": finished loading " +name);
	return newClass;
    }

    /*
     * Simple loader, just look for the .class file in the current
     * directory.
     */
    protected byte[] bytesForClass(String name)
	throws IOException
    {
	FileInputStream in = new FileInputStream(name+ ".class");
	int length = in.available();
	byte[] buf = new byte[length];
	in.read(buf);
	return buf;
    }
}
