/*
 * TV.java
 *
 * This test is based on one distributed with Kaffe.
 */
import java.util.Random;

public class TV 
{
	public static void main(String[] args)
	{
		int n = 5000;

		if (args.length > 0)
		{
			n = Integer.parseInt(args[0]);
		}
		System.out.println(n + " iterations.");

		TV_Vec[] r = new TV_Vec[n], a = new TV_Vec[n], b = new TV_Vec[n];

		System.out.println("Allocated vector vectors");

		Random random = new Random(89071);

		System.out.println("Allocated random number generator");

		for (int i = 0; i < n; i++)
		{
			a[i] = new TV_Vec(
				random.nextDouble(),
				random.nextDouble(),
				random.nextDouble());
			b[i] = new TV_Vec(
				random.nextDouble(),
				random.nextDouble(),
				random.nextDouble());
		}

		System.out.println("Starting...");

		long start = System.currentTimeMillis();
		for (int i = 0; i < n; i++)
		{
			r[i] = TV_Vec.cross(a[i] , b[i]);
		}
		long stop = System.currentTimeMillis();

		// Surrond the number in "***" "***" so that a smart test
		// suite will ignore them.
		System.out.println("That took ***" + (stop - start) + "*** milliseconds.");
	}
}

class TV_Vec {
	public TV_Vec()
	{
		v = new double[3];
		v[0] = v[1] = v[2] = 0.0;
	}
	public TV_Vec(double v0, double v1, double v2)
	{
		v = new double[3];
		v[0] = v0;
		v[1] = v1;
		v[2] = v2;
	}
	public TV_Vec(double[] _v)
	{
		v = _v;
	}
	public String toString()
	{
		return "( " + v[0] + ", " + v[1] + ", " + v[2] + " )";
	}
	public static double dot(TV_Vec p, TV_Vec q)
	{
		return
			p.v[0] * q.v[0] +
			p.v[1] * q.v[1] +
			p.v[2] * q.v[2];
	}
	public static TV_Vec cross(TV_Vec p, TV_Vec q)
	{
		return new TV_Vec(
			p.v[1] * q.v[2] - p.v[2] * q.v[1],
			p.v[2] * q.v[0] - p.v[0] * q.v[2],
			p.v[0] * q.v[1] - p.v[1] * q.v[0]);
	}
	public static double min(TV_Vec p)
	{
		return Math.min(Math.min(p.v[0], p.v[1]), p.v[2]);
	}
	public static double max(TV_Vec p)
	{
		return Math.max(Math.max(p.v[0], p.v[1]), p.v[2]);
	}
	public static TV_Vec vmax(TV_Vec p, TV_Vec q)
	{
		return new TV_Vec(
			Math.max(p.v[0], q.v[0]),
			Math.max(p.v[1], q.v[1]),
			Math.max(p.v[2], q.v[2]));
	}
	public static TV_Vec vmin(TV_Vec p, TV_Vec q)
	{
		return new TV_Vec(
			Math.min(p.v[0], q.v[0]),
			Math.min(p.v[1], q.v[1]),
			Math.min(p.v[2], q.v[2]));
	}
	public static double length(TV_Vec p)
	{
		return Math.sqrt(p.v[0] * p.v[0] + p.v[1] * p.v[1] + p.v[2] * p.v[2]);
	}
	public static TV_Vec add(TV_Vec p, TV_Vec q)
	{
		return new TV_Vec(
			p.v[0] + q.v[0],
			p.v[1] + q.v[1],
			p.v[2] + q.v[2]);
	}
	public static TV_Vec neg(TV_Vec p)
	{
		return new TV_Vec(
			-p.v[0],
			-p.v[1],
			-p.v[2]);
	}
	public static TV_Vec sub(TV_Vec p, TV_Vec q)
	{
		return new TV_Vec(
			p.v[0] - q.v[0],
			p.v[1] - q.v[1],
			p.v[2] - q.v[2]);
	}
	public static TV_Vec mul(double p, TV_Vec q)
	{
		return new TV_Vec(
			p * q.v[0],
			p * q.v[1],
			p * q.v[2]);
	}
	public static TV_Vec mul(TV_Vec p, double q)
	{
		return new TV_Vec(
			p.v[0] * q,
			p.v[1] * q,
			p.v[2] * q);
	}
	public static TV_Vec div(TV_Vec p, double q)
	{
		double r = 1.0 / q;
		return new TV_Vec(
			p.v[0] * r,
			p.v[1] * r,
			p.v[2] * r);
	}
	public double elt(int i)
	{
		return v[i];
	}
	public double[] elts()
	{
		return v;
	}
	protected double[] v;
};
