//
// Synchro.java
//
// Do some test of the overhead of synchronization.  Its
// hard to read the output and get a feeling for that
// overhead, though.
//

class SynchroData
{
	int[] ids_;
	int ct_;

	public SynchroData(int ct) 
	{
		ids_ = new int[ct];
		ct_ = ct;

		reset();
	}
	
	public synchronized void access_synch(int id)
	{
		ids_[id]++;
	}
	
	public void access_nosynch(int id)
	{
		ids_[id]++;
	}
	
	public void results()
	{
		int i;
		
		for (i =0; i< ct_; i++)
			System.out.println((i+1)+ ": " +ids_[i]);
	}

	public void reset()
	{
		int i;

		for (i =0; i< ct_; i++)
			ids_[i] = 0;
	}
}


class Synchro_TestThread 
	implements Runnable
{
	SynchroData synchobj_;
	int id_;

	public Synchro_TestThread(SynchroData synchobj, int id) 
	{
		synchobj_ = synchobj;
		id_ = id;
	}

	public void run()
	{
		if ((id_ % 2) == 0)
			while (true)
				synchobj_.access_synch(id_);
		else
			while (true)
				synchobj_.access_nosynch(id_);
	}
}


public class Synchro
{
	static void pause(int sec)
	{
		try 
		{
			Thread.sleep(sec * 1000);
		} 
		catch (java.lang.InterruptedException e)
		{
			System.out.println("Try again, sleeping thread interrupted.");
		}
	}
	

	public static void main(String[] args)
	{
		int ct = 10;
		int pauseSec = 10;
		Thread newT[] = new Thread[ct];
		int i;
		SynchroData sobj = new SynchroData(ct);
		
		Thread.currentThread().setPriority(Thread.MAX_PRIORITY);

		System.out.println("Firing off threads [1.." +ct+ "] for " +pauseSec+ " seconds.");
		// Create all of the threads
		for (i=0; i<ct; i++)
		{
			newT[i] = new Thread(new Synchro_TestThread(sobj, i));
			newT[i].setPriority(Thread.MIN_PRIORITY);
		}
		
		// Fire em off
		for (i=0; i<ct; i++)
			newT[i].start();
		pause(2);
		sobj.reset();
		
		pause(pauseSec);
		
		// Kill em
		for (i=0; i<ct; i++)
			newT[i].stop();

		sobj.results();

		sobj.reset();
		System.out.println("Firing off threads [" +ct+ "..1] for " +pauseSec+ " seconds.");
		for (i=0; i<ct; i++)
		{
			newT[i] = new Thread(new Synchro_TestThread(sobj, i));
			newT[i].setPriority(Thread.MIN_PRIORITY);
		}
		
		for (i=ct-1; i>=0; i--) // Notice the reverse order
			newT[i].start();
		pause(2);
		sobj.reset();
		pause(pauseSec);
		for (i=ct-1; i>=0; i--) // Notice the reverse order
			newT[i].stop();
		sobj.results();
		
		Runtime.getRuntime().exit(0);
	}
}
