
// (C) 1997 Glynn Clements <glynn@sensei.co.uk> - Freely Redistributable

package java.awt;

import java.awt.peer.ContainerPeer;

import java.io.PrintStream;

import java.util.Vector;
import java.util.Enumeration;

public abstract class Container extends Component
{
	Vector children = new Vector();
	LayoutManager layout;

	protected String paramString()
	{
		StringBuffer buff = new StringBuffer();
		buff.append(super.paramString());
		buff.append(", children={");

		Enumeration e = children.elements();
		while (e.hasMoreElements())
		{
			Component comp = (Component) e.nextElement();
			buff.append(comp);
			if (e.hasMoreElements())
				buff.append(", ");
		}

		buff.append("}");

		if (layout != null)
			buff.append(", layout=" + layout);

		return buff.toString();
	}

	public void addNotify()
	{
		Enumeration e = children.elements();
		while (e.hasMoreElements())
		{
			Component comp = (Component) e.nextElement();
			comp.addNotify();
		}

		super.addNotify();
	}

	public void removeNotify()
	{
		Enumeration e = children.elements();
		while (e.hasMoreElements())
		{
			Component comp = (Component) e.nextElement();
			comp.removeNotify();
		}

		super.removeNotify();
	}

	public void setLayout(LayoutManager layout)
	{
		this.layout = layout;
	}

	public Component add(Component comp, int index)
	{
		if (index == -1)
			children.addElement(comp);
		else
			children.insertElementAt(comp, index);

		comp.parent = this;

		return comp;
	}

	public Component add(Component comp)
	{
		return add(comp, -1);
	}

	public Component add(String name, Component comp)
	{
		layout.addLayoutComponent(name, comp);
		return add(comp);
	}

	public void remove(Component comp)
	{
		children.removeElement(comp);
		comp.removeNotify();
	}

	public void removeAll()
	{
		Enumeration e = children.elements();
		while (e.hasMoreElements())
		{
			Component comp = (Component) e.nextElement();
			remove(comp);
		}
	}

	public LayoutManager getLayout()
	{
		return layout;
	}

	public int countComponents()
	{
		return children.size();
	}

	public Component getComponent(int n)
	{
		return (Component) children.elementAt(n);
	}

	public Component[] getComponents()
	{
		int n = countComponents();
		Component[] result = new Component[n];
		for (int i = 0; i < result.length; i++)
			result[i] = getComponent(i);
		return result;
	}

	public Insets insets()
	{
		if (peer != null)
			return ((ContainerPeer) peer).insets();
		return new Insets(0, 0, 0, 0);
	}

	public Dimension minimumSize()
	{
		if (layout != null)
			return layout.minimumLayoutSize(this);
		else
			return super.minimumSize();
	}

	public Dimension preferredSize()
	{
		if (layout != null)
			return layout.preferredLayoutSize(this);
		else
			return super.preferredSize();
	}

	public Component locate(int x, int y)
	{
		if (!inside(x, y))
			return null;

		Enumeration e = children.elements();
		while (e.hasMoreElements())
		{
			Component comp = (Component) e.nextElement();
			if (comp.inside(x, y))
				return comp;
		}

		return this;
	}

	public void layout()
	{
		layout.layoutContainer(this);
	}

	public void validate()
	{
		super.validate();
		Enumeration e = children.elements();
		while (e.hasMoreElements())
		{
			Component comp = (Component) e.nextElement();
			comp.validate();
		}
	}

	public void deliverEvent(Event e)
	{
		Component comp = locate(e.x, e.y);
		if (comp == null)
			postEvent(e);
		e.translate(-comp.x, -comp.y);
		comp.deliverEvent(e);
	}

	public void paintComponents(Graphics g)
	{
		Enumeration e = children.elements();
		while (e.hasMoreElements())
		{
			Component comp = (Component) e.nextElement();
			Graphics g2 = g.create(comp.x, comp.y, comp.width, comp.height);
			comp.paint(g2);
		}
	}

	public void printComponents(Graphics g)
	{
		Enumeration e = children.elements();
		while (e.hasMoreElements())
		{
			Component comp = (Component) e.nextElement();
			Graphics g2 = g.create(comp.x, comp.y, comp.width, comp.height);
			comp.print(g2);
		}
	}

	public void list(PrintStream out, int indent)
	{
		super.list(out, indent);
		indent++;
		Enumeration e = children.elements();
		while (e.hasMoreElements())
		{
			Component comp = (Component) e.nextElement();
			comp.list(out, indent);
		}
	}
}
