
// (C) 1997 Glynn Clements <glynn@sensei.co.uk> - Freely Redistributable

package java.awt;

public class Event
{
	public final static int WINDOW_DESTROY		= 201;
	public final static int WINDOW_EXPOSE		= 202;
	public final static int WINDOW_ICONIFY		= 203;
	public final static int WINDOW_DEICONIFY	= 204;
	public final static int WINDOW_MOVED		= 205;
	public final static int KEY_PRESS		= 401;
	public final static int KEY_RELEASE		= 402;
	public final static int KEY_ACTION		= 403;
	public final static int KEY_ACTION_RELEASE	= 404;
	public final static int MOUSE_DOWN		= 501;
	public final static int MOUSE_UP		= 502;
	public final static int MOUSE_MOVE		= 503;
	public final static int MOUSE_ENTER		= 504;
	public final static int MOUSE_EXIT		= 505;
	public final static int MOUSE_DRAG		= 506;
	public final static int SCROLL_LINE_UP		= 601;
	public final static int SCROLL_LINE_DOWN	= 602;
	public final static int SCROLL_PAGE_UP		= 603;
	public final static int SCROLL_PAGE_DOWN	= 604;
	public final static int SCROLL_ABSOLUTE		= 605;
	public final static int LIST_SELECT		= 701;
	public final static int LIST_DESELECT		= 702;
	public final static int ACTION_EVENT		= 1001;
	public final static int LOAD_FILE		= 1002;
	public final static int SAVE_FILE		= 1003;
	public final static int GOT_FOCUS		= 1004;
	public final static int LOST_FOCUS		= 1005;

	public final static int HOME		= 1000;
	public final static int END		= 1001;
	public final static int PGUP		= 1002;
	public final static int PGDN		= 1003;
	public final static int UP		= 1004;
	public final static int DOWN		= 1005;
	public final static int LEFT		= 1006;
	public final static int RIGHT		= 1007;
	public final static int F1		= 1008;
	public final static int F2		= 1009;
	public final static int F3		= 1010;
	public final static int F4		= 1011;
	public final static int F5		= 1012;
	public final static int F6		= 1013;
	public final static int F7		= 1014;
	public final static int F8		= 1015;
	public final static int F9		= 1016;
	public final static int F10		= 1017;
	public final static int F11		= 1018;
	public final static int F12		= 1019;

	public final static int SHIFT_MASK	= 1;
	public final static int CTRL_MASK	= 2;
	public final static int META_MASK	= 4;
	public final static int ALT_MASK	= 8;

	private final static String[] modStrings = {
		"....", "...S", "..C.", "..CS",
		".M..", ".M.S", ".MC.", ".MCS",
		"A...", "A..S", "A.C.", "A.CS",
		"AM..", "AM.S", "AMC.", "AMCS",
	};

	public Object arg;
	public Event evt;
	public int id;
	public int key;
	public int modifiers;
	public Object target;
	public long when;
	public int x;
	public int y;
	public int clickCount;

	protected String paramString()
	{
		return	"Event:target=" + target +
			", time=" + when +
			", ID=" + id +
			", pos=(" + x + ", " + y + ")" +
			", mod=" + modStrings[modifiers] +
			", key=" + key +
			", arg=" + arg;
	}

	public String toString()
	{
		return paramString();
	}

	public boolean shiftDown()
	{
		return (modifiers & SHIFT_MASK) != 0;
	}

	public boolean controlDown()
	{
		return (modifiers & CTRL_MASK) != 0;
	}

	public boolean metaDown()
	{
		return (modifiers & META_MASK) != 0;
	}

	public void translate(int dx, int dy)
	{
		x += dx;
		y += dy;
	}

	public Event(Object target, long when, int id, int x, int y, int key, int modifiers, Object arg)
	{
		this.target	= target;
		this.when	= when;
		this.id		= id;
		this.x		= x;
		this.y		= y;
		this.key	= key;
		this.modifiers	= modifiers;
		this.arg	= arg;
	}

	public Event(Object target, long when, int id, int x, int y, int key, int modifiers)
	{
		this(target, when, id, x, y, key, modifiers, null);
	}

	public Event(Object target, int id, Object arg)
	{
		this(target, System.currentTimeMillis(), id, 0, 0, 0, 0, arg);
	}
}

