
// (C) 1997 Glynn Clements <glynn@sensei.co.uk> - Freely Redistributable

package java.awt;

import java.awt.peer.FramePeer;
import java.awt.peer.MenuBarPeer;

public class Frame extends Window implements MenuContainer
{
	public final static int DEFAULT_CURSOR		= 0;
	public final static int CROSSHAIR_CURSOR	= 1;
	public final static int TEXT_CURSOR		= 2;
	public final static int WAIT_CURSOR		= 3;
	public final static int SW_RESIZE_CURSOR	= 4;
	public final static int SE_RESIZE_CURSOR	= 5;
	public final static int NW_RESIZE_CURSOR	= 6;
	public final static int NE_RESIZE_CURSOR	= 7;
	public final static int N_RESIZE_CURSOR		= 8;
	public final static int S_RESIZE_CURSOR		= 9;
	public final static int W_RESIZE_CURSOR		= 10;
	public final static int E_RESIZE_CURSOR		= 11;
	public final static int HAND_CURSOR		= 12;
	public final static int MOVE_CURSOR		= 13;

	private String title;
	private boolean resizable;
	private MenuBar menubar;
	private int cursor;
	private Image icon;

	protected String paramString()
	{
		return	super.paramString() +
			", title=" + title +
			", resizable=" + resizable +
			", menubar=" + menubar +
			", cursor=" + cursor +
			", icon=" + icon;
	}

	public void addNotify()
	{
		peer = getToolkit().createFrame(this);
	}

	public void remove(MenuComponent m)
	{
		if (m != menubar)
			return;

		MenuBarPeer mp = (MenuBarPeer) menubar.getPeer();
		if (mp != null)
			mp.dispose();

		if (peer != null)
			((FramePeer) peer).setMenuBar(null);

		this.menubar = null;
	}

	public void dispose()
	{
		if (menubar != null)
			remove(menubar);
		menubar = null;

		if (peer != null)
			peer.dispose();
		peer = null;

		super.dispose();
	}

	public void setTitle(String title)
	{
		this.title = title;
		if (peer != null)
			((FramePeer) peer).setTitle(title);
	}

	public void setResizable(boolean resizable)
	{
		this.resizable = resizable;
		if (peer != null)
			((FramePeer) peer).setResizable(resizable);
	}

	public void setMenuBar(MenuBar menubar)
	{
		this.menubar = menubar;
		if (peer != null)
			((FramePeer) peer).setMenuBar(menubar);
	}

	public void setCursor(int cursor)
	{
		this.cursor = cursor;
		if (peer != null)
			((FramePeer) peer).setCursor(cursor);
	}

	public void setIconImage(Image icon)
	{
		this.icon = icon;
		if (peer != null)
			((FramePeer) peer).setIconImage(icon);
	}

	public String getTitle()
	{
		return title;
	}

	public boolean isResizable()
	{
		return resizable;
	}

	public MenuBar getMenuBar()
	{
		return menubar;
	}

	public int getCursorType()
	{
		return cursor;
	}

	public Image getIconImage()
	{
		return icon;
	}

	public Frame(String title)
	{
		super(null);
		this.title = title;
	}

	public Frame()
	{
		this(null);
	}
}
