
// (C) 1997 Glynn Clements <glynn@sensei.co.uk> - Freely Redistributable

package java.awt;

import java.awt.peer.ListPeer;

import java.util.Vector;
import java.util.Enumeration;

public class List extends Component
{
	private Vector list = new Vector();
	private boolean multi = false;
	private int rows = 0;
	private int visible = 0;

	protected String paramString()
	{
		StringBuffer buff = new StringBuffer();
		buff.append(super.paramString());
		buff.append(", items={");

		Enumeration e = list.elements();
		while (e.hasMoreElements())
		{
			String item = (String) e.nextElement();
			buff.append(item);
			if (e.hasMoreElements())
				buff.append(", ");
		}

		buff.append("}");

		if (multi)
			buff.append(", multi");
		if (rows > 0)
			buff.append(", rows=" + rows);
		buff.append(", visible=" + visible);

		return buff.toString();
	}

	public void addNotify()
	{
		peer = getToolkit().createList(this);
	}

	public void removeNotify()
	{
		if (peer != null)
			peer.dispose();
		peer = null;
	}

	public void setMultipleSelections(boolean multi)
	{
		this.multi = multi;
	}

	public void clear()
	{
		list = new Vector();
		if (peer != null)
			((ListPeer) peer).delItems(0, countItems());
	}

	public void addItem(String item, int index)
	{
		list.addElement(item);
		if (peer != null)
			((ListPeer) peer).addItem(item, index);
	}

	public void addItem(String item)
	{
		addItem(item, -1);
	}

	public void replaceItem(String newValue, int index)
	{
		delItem(index);
		addItem(newValue, index);
	}

	public void delItems(int start, int end)
	{
		for (int i = start; i <= end; i++)
			list.removeElementAt(i);
		if (peer != null)
			((ListPeer) peer).delItems(start, end);
	}

	public void delItem(int index)
	{
		list.removeElementAt(index);
		if (peer != null)
			((ListPeer) peer).delItems(index, index);
	}

	public void select(int index)
	{
		if (peer != null)
			((ListPeer) peer).select(index);
	}

	public void deselect(int index)
	{
		if (peer != null)
			((ListPeer) peer).deselect(index);
	}

	public void makeVisible(int index)
	{
		visible = index;
		if (peer != null)
			((ListPeer) peer).select(index);
	}

	public int countItems()
	{
		return list.size();
	}

	public String getItem(int index)
	{
		return (String) list.elementAt(index);
	}

	public int[] getSelectedIndexes()
	{
		return	(peer != null)
			? ((ListPeer) peer).getSelectedIndexes()
			: new int[0];
	}

	public int getSelectedIndex()
	{
		int[] indices = getSelectedIndexes();

		if (indices.length == 1)
			return indices[0];
		else
			return -1;
	}

	public String getSelectedItem()
	{
		int n = getSelectedIndex();
		return (n >= 0) ? getItem(n) : null;
	}

	public String[] getSelectedItems()
	{
		int[] indices = getSelectedIndexes();
		int n = indices.length;

		String[] result = new String[n];
		for (int i = 0; i < n; i++)
			result[i] = getItem(indices[i]);

		return result;
	}

	public boolean isSelected(int index)
	{
		int[] indices = getSelectedIndexes();
		int n = indices.length;

		for (int i = 0; i < n; i++)
			if (indices[i] == index)
				return true;

		return false;
	}

	public boolean allowsMultipleSelections()
	{
		return multi;
	}

	public int getVisibleIndex()
	{
		return visible;
	}

	public int getRows()
	{
		return rows;
	}

	public Dimension minimumSize()
	{
		return	(peer != null && rows > 0)
			? ((ListPeer) peer).minimumSize(rows)
			: super.minimumSize();
	}

	public Dimension minimumSize(int rows)
	{
		return	(peer != null)
			? ((ListPeer) peer).minimumSize(rows)
			: super.minimumSize();
	}

	public Dimension preferredSize()
	{
		return	(peer != null && rows > 0)
			? ((ListPeer) peer).preferredSize(rows)
			: super.preferredSize();
	}

	public Dimension preferredSize(int rows)
	{
		return	(peer != null)
			? ((ListPeer) peer).preferredSize(rows)
			: super.preferredSize();
	}

	public List(int rows, boolean multi)
	{
		this.rows = rows;
		this.multi = multi;
	}

	public List()
	{
		this(0, false);
	}
}

